@@ -1,5 +1,5 @@
 /* <uchar.h> substitute - 16-bit and 32-bit wide character types.
-   Copyright (C) 2019-2024 Free Software Foundation, Inc.
+   Copyright (C) 2019-2023 Free Software Foundation, Inc.
 
    This file is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
@@ -21,13 +21,13 @@
  */
 
 #ifndef _@GUARD_PREFIX@_UCHAR_H
+#define _@GUARD_PREFIX@_UCHAR_H
 
 #if __GNUC__ >= 3
 @PRAGMA_SYSTEM_HEADER@
 #endif
 @PRAGMA_COLUMNS@
 
-/* The include_next requires a split double-inclusion guard.  */
 #if @HAVE_UCHAR_H@
 # if defined __HAIKU__
 /* Work around <https://dev.haiku-os.org/ticket/17040>.  */
@@ -44,9 +44,6 @@
 # @INCLUDE_NEXT@ @NEXT_UCHAR_H@
 #endif
 
-#ifndef _@GUARD_PREFIX@_UCHAR_H
-#define _@GUARD_PREFIX@_UCHAR_H
-
 /* This file uses _GL_INLINE_HEADER_BEGIN, _GL_INLINE, _GL_BEGIN_C_LINKAGE,
    _GL_ATTRIBUTE_PURE, GNULIB_POSIXCHECK, HAVE_RAW_DECL_*.  */
 #if !_GL_CONFIG_H_INCLUDED
@@ -162,15 +159,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE _GL_ATTRIBUTE_PURE wint_t
 btoc32 (int c)
 {
-  return
-#  if @GNULIB_BTOWC@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         btowc (c);
+  return btowc (c);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (btoc32, wint_t, (int c), _GL_ATTRIBUTE_PURE);
+_GL_FUNCDECL_SYS (btoc32, wint_t, (int c) _GL_ATTRIBUTE_PURE);
 # endif
 _GL_CXXALIAS_SYS (btoc32, wint_t, (int c));
 _GL_CXXALIASWARN (btoc32);
@@ -184,15 +177,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isalnum (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswalnum (wc);
+  return iswalnum (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isalnum, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isalnum, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isalnum, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isalnum);
@@ -203,15 +192,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isalpha (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswalpha (wc);
+  return iswalpha (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isalpha, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isalpha, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isalpha, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isalpha);
@@ -222,15 +207,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isblank (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswblank (wc);
+  return iswblank (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isblank, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isblank, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isblank, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isblank);
@@ -241,15 +222,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32iscntrl (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswcntrl (wc);
+  return iswcntrl (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32iscntrl, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32iscntrl, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32iscntrl, int, (wint_t wc));
 _GL_CXXALIASWARN (c32iscntrl);
@@ -260,15 +237,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isdigit (wint_t wc)
 {
-  return
-#  if @GNULIB_ISWDIGIT@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswdigit (wc);
+  return iswdigit (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isdigit, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isdigit, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isdigit, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isdigit);
@@ -279,15 +252,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isgraph (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswgraph (wc);
+  return iswgraph (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isgraph, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isgraph, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isgraph, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isgraph);
@@ -298,15 +267,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32islower (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswlower (wc);
+  return iswlower (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32islower, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32islower, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32islower, int, (wint_t wc));
 _GL_CXXALIASWARN (c32islower);
@@ -317,15 +282,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isprint (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswprint (wc);
+  return iswprint (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isprint, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isprint, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isprint, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isprint);
@@ -336,15 +297,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32ispunct (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswpunct (wc);
+  return iswpunct (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32ispunct, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32ispunct, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32ispunct, int, (wint_t wc));
 _GL_CXXALIASWARN (c32ispunct);
@@ -355,15 +312,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isspace (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswspace (wc);
+  return iswspace (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isspace, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isspace, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isspace, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isspace);
@@ -374,15 +327,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isupper (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswupper (wc);
+  return iswupper (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isupper, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isupper, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isupper, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isupper);
@@ -393,15 +342,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32isxdigit (wint_t wc)
 {
-  return
-#  if @GNULIB_ISWXDIGIT@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswxdigit (wc);
+  return iswxdigit (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32isxdigit, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32isxdigit, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32isxdigit, int, (wint_t wc));
 _GL_CXXALIASWARN (c32isxdigit);
@@ -415,15 +360,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE wint_t
 c32tolower (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         towlower (wc);
+  return towlower (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32tolower, wint_t, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32tolower, wint_t, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32tolower, wint_t, (wint_t wc));
 _GL_CXXALIASWARN (c32tolower);
@@ -434,15 +375,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE wint_t
 c32toupper (wint_t wc)
 {
-  return
-#  if defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         towupper (wc);
+  return towupper (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32toupper, wint_t, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32toupper, wint_t, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32toupper, wint_t, (wint_t wc));
 _GL_CXXALIASWARN (c32toupper);
@@ -456,15 +393,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32width (char32_t wc)
 {
-  return
-#  if @GNULIB_WCWIDTH@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         wcwidth (wc);
+  return wcwidth (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32width, int, (char32_t wc), );
+_GL_FUNCDECL_SYS (c32width, int, (char32_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32width, int, (char32_t wc));
 _GL_CXXALIASWARN (c32width);
@@ -478,11 +411,11 @@ _GL_CXXALIASWARN (c32width);
 #   undef c32rtomb
 #   define c32rtomb rpl_c32rtomb
 #  endif
-_GL_FUNCDECL_RPL (c32rtomb, size_t, (char *s, char32_t wc, mbstate_t *ps), );
+_GL_FUNCDECL_RPL (c32rtomb, size_t, (char *s, char32_t wc, mbstate_t *ps));
 _GL_CXXALIAS_RPL (c32rtomb, size_t, (char *s, char32_t wc, mbstate_t *ps));
 # else
 #  if !@HAVE_C32RTOMB@
-_GL_FUNCDECL_SYS (c32rtomb, size_t, (char *s, char32_t wc, mbstate_t *ps), );
+_GL_FUNCDECL_SYS (c32rtomb, size_t, (char *s, char32_t wc, mbstate_t *ps));
 #  endif
 _GL_CXXALIAS_SYS (c32rtomb, size_t, (char *s, char32_t wc, mbstate_t *ps));
 # endif
@@ -506,17 +439,13 @@ _GL_INLINE _GL_ARG_NONNULL ((2)) size_t
 c32snrtombs (char *dest, const char32_t **srcp, size_t srclen, size_t len,
              mbstate_t *ps)
 {
-  return
-#  if @GNULIB_WCSNRTOMBS@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         wcsnrtombs (dest, (const wchar_t **) srcp, srclen, len, ps);
+  return wcsnrtombs (dest, (const wchar_t **) srcp, srclen, len, ps);
 }
 _GL_END_C_LINKAGE
 # else
 _GL_FUNCDECL_SYS (c32snrtombs, size_t,
                   (char *dest, const char32_t **srcp, size_t srclen, size_t len,
-                   mbstate_t *ps),
+                   mbstate_t *ps)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (c32snrtombs, size_t,
@@ -533,17 +462,12 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE _GL_ARG_NONNULL ((2)) size_t
 c32srtombs (char *dest, const char32_t **srcp, size_t len, mbstate_t *ps)
 {
-  return
-#  if @GNULIB_WCSRTOMBS@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         wcsrtombs (dest, (const wchar_t **) srcp, len, ps);
+  return wcsrtombs (dest, (const wchar_t **) srcp, len, ps);
 }
 _GL_END_C_LINKAGE
 # else
 _GL_FUNCDECL_SYS (c32srtombs, size_t,
-                  (char *dest, const char32_t **srcp, size_t len,
-                   mbstate_t *ps),
+                  (char *dest, const char32_t **srcp, size_t len, mbstate_t *ps)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (c32srtombs, size_t,
@@ -568,7 +492,7 @@ c32stombs (char *dest, const char32_t *src, size_t len)
 _GL_END_C_LINKAGE
 # else
 _GL_FUNCDECL_SYS (c32stombs, size_t,
-                  (char *dest, const char32_t *src, size_t len),
+                  (char *dest, const char32_t *src, size_t len)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (c32stombs, size_t,
@@ -584,15 +508,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE _GL_ARG_NONNULL ((1)) int
 c32swidth (const char32_t *s, size_t n)
 {
-  return
-#  if @GNULIB_WCSWIDTH@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         wcswidth ((const wchar_t *) s, n);
+  return wcswidth ((const wchar_t *) s, n);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32swidth, int, (const char32_t *s, size_t n),
+_GL_FUNCDECL_SYS (c32swidth, int, (const char32_t *s, size_t n)
                                   _GL_ARG_NONNULL ((1)));
 # endif
 _GL_CXXALIAS_SYS (c32swidth, int, (const char32_t *s, size_t n));
@@ -609,15 +529,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32tob (wint_t wc)
 {
-  return
-#  if @GNULIB_WCTOB@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         wctob (wc);
+  return wctob (wc);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32tob, int, (wint_t wc), );
+_GL_FUNCDECL_SYS (c32tob, int, (wint_t wc));
 # endif
 _GL_CXXALIAS_SYS (c32tob, int, (wint_t wc));
 _GL_CXXALIASWARN (c32tob);
@@ -632,13 +548,13 @@ _GL_CXXALIASWARN (c32tob);
 #   define mbrtoc32 rpl_mbrtoc32
 #  endif
 _GL_FUNCDECL_RPL (mbrtoc32, size_t,
-                  (char32_t *pc, const char *s, size_t n, mbstate_t *ps), );
+                  (char32_t *pc, const char *s, size_t n, mbstate_t *ps));
 _GL_CXXALIAS_RPL (mbrtoc32, size_t,
                   (char32_t *pc, const char *s, size_t n, mbstate_t *ps));
 # else
 #  if !@HAVE_MBRTOC32@
 _GL_FUNCDECL_SYS (mbrtoc32, size_t,
-                  (char32_t *pc, const char *s, size_t n, mbstate_t *ps), );
+                  (char32_t *pc, const char *s, size_t n, mbstate_t *ps));
 #  endif
 _GL_CXXALIAS_SYS (mbrtoc32, size_t,
                   (char32_t *pc, const char *s, size_t n, mbstate_t *ps));
@@ -664,13 +580,13 @@ _GL_WARN_ON_USE (mbrtoc32, "mbrtoc32 is not portable - "
 #   define mbrtoc16 rpl_mbrtoc16
 #  endif
 _GL_FUNCDECL_RPL (mbrtoc16, size_t,
-                  (char16_t *pc, const char *s, size_t n, mbstate_t *ps), );
+                  (char16_t *pc, const char *s, size_t n, mbstate_t *ps));
 _GL_CXXALIAS_RPL (mbrtoc16, size_t,
                   (char16_t *pc, const char *s, size_t n, mbstate_t *ps));
 # else
-#  if !@HAVE_MBRTOC16@
+#  if !@HAVE_MBRTOC32@
 _GL_FUNCDECL_SYS (mbrtoc16, size_t,
-                  (char16_t *pc, const char *s, size_t n, mbstate_t *ps), );
+                  (char16_t *pc, const char *s, size_t n, mbstate_t *ps));
 #  endif
 _GL_CXXALIAS_SYS (mbrtoc16, size_t,
                   (char16_t *pc, const char *s, size_t n, mbstate_t *ps));
@@ -695,17 +611,13 @@ _GL_INLINE _GL_ARG_NONNULL ((2)) size_t
 mbsnrtoc32s (char32_t *dest, const char **srcp, size_t srclen, size_t len,
              mbstate_t *ps)
 {
-  return
-#  if @GNULIB_MBSNRTOWCS@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         mbsnrtowcs ((wchar_t *) dest, srcp, srclen, len, ps);
+  return mbsnrtowcs ((wchar_t *) dest, srcp, srclen, len, ps);
 }
 _GL_END_C_LINKAGE
 # else
 _GL_FUNCDECL_SYS (mbsnrtoc32s, size_t,
                   (char32_t *dest, const char **srcp, size_t srclen, size_t len,
-                   mbstate_t *ps),
+                   mbstate_t *ps)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (mbsnrtoc32s, size_t,
@@ -722,17 +634,12 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE _GL_ARG_NONNULL ((2)) size_t
 mbsrtoc32s (char32_t *dest, const char **srcp, size_t len, mbstate_t *ps)
 {
-  return
-#  if @GNULIB_MBSRTOWCS@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         mbsrtowcs ((wchar_t *) dest, srcp, len, ps);
+  return mbsrtowcs ((wchar_t *) dest, srcp, len, ps);
 }
 _GL_END_C_LINKAGE
 # else
 _GL_FUNCDECL_SYS (mbsrtoc32s, size_t,
-                  (char32_t *dest, const char **srcp, size_t len,
-                   mbstate_t *ps),
+                  (char32_t *dest, const char **srcp, size_t len, mbstate_t *ps)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (mbsrtoc32s, size_t,
@@ -757,7 +664,7 @@ mbstoc32s (char32_t *dest, const char *src, size_t len)
 _GL_END_C_LINKAGE
 # else
 _GL_FUNCDECL_SYS (mbstoc32s, size_t,
-                  (char32_t *dest, const char *src, size_t len),
+                  (char32_t *dest, const char *src, size_t len)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (mbstoc32s, size_t,
@@ -790,15 +697,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE _GL_ARG_NONNULL ((1)) c32_type_test_t
 c32_get_type_test (const char *name)
 {
-  return
-#  if @GNULIB_WCTYPE@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         wctype (name);
+  return wctype (name);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32_get_type_test, c32_type_test_t, (const char *name),
+_GL_FUNCDECL_SYS (c32_get_type_test, c32_type_test_t, (const char *name)
                                                       _GL_ARG_NONNULL ((1)));
 # endif
 _GL_CXXALIAS_SYS (c32_get_type_test, c32_type_test_t, (const char *name));
@@ -817,20 +720,16 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE int
 c32_apply_type_test (wint_t wc, c32_type_test_t property)
 {
-  return
-#  if @GNULIB_ISWCTYPE@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         iswctype (wc, property);
+  return iswctype (wc, property);
 }
 _GL_END_C_LINKAGE
 #  else
 _GL_FUNCDECL_SYS (c32_apply_type_test, int,
-                  (wint_t wc, c32_type_test_t property), );
+                  (wint_t wc, c32_type_test_t property));
 #  endif
 # else
 _GL_FUNCDECL_SYS (c32_apply_type_test, int,
-                  (wint_t wc, c32_type_test_t property),
+                  (wint_t wc, c32_type_test_t property)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (c32_apply_type_test, int,
@@ -862,15 +761,11 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE _GL_ARG_NONNULL ((1)) c32_mapping_t
 c32_get_mapping (const char *name)
 {
-  return
-#  if @GNULIB_WCTRANS@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         wctrans (name);
+  return wctrans (name);
 }
 _GL_END_C_LINKAGE
 # else
-_GL_FUNCDECL_SYS (c32_get_mapping, c32_mapping_t, (const char *name),
+_GL_FUNCDECL_SYS (c32_get_mapping, c32_mapping_t, (const char *name)
                                                   _GL_ARG_NONNULL ((1)));
 # endif
 _GL_CXXALIAS_SYS (c32_get_mapping, c32_mapping_t, (const char *name));
@@ -888,16 +783,12 @@ _GL_BEGIN_C_LINKAGE
 _GL_INLINE _GL_ARG_NONNULL ((2)) wint_t
 c32_apply_mapping (wint_t wc, c32_mapping_t mapping)
 {
-  return
-#  if @GNULIB_TOWCTRANS@ && defined __cplusplus && defined GNULIB_NAMESPACE
-         GNULIB_NAMESPACE::
-#  endif
-         towctrans (wc, mapping);
+  return towctrans (wc, mapping);
 }
 _GL_END_C_LINKAGE
 # else
 _GL_FUNCDECL_SYS (c32_apply_mapping, wint_t,
-                  (wint_t wc, c32_mapping_t mapping),
+                  (wint_t wc, c32_mapping_t mapping)
                   _GL_ARG_NONNULL ((2)));
 # endif
 _GL_CXXALIAS_SYS (c32_apply_mapping, wint_t,
@@ -909,4 +800,3 @@ _GL_CXXALIASWARN (c32_apply_mapping);
 _GL_INLINE_HEADER_END
 
 #endif /* _@GUARD_PREFIX@_UCHAR_H */
-#endif /* _@GUARD_PREFIX@_UCHAR_H */
