@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -45,10 +45,10 @@
 static const unsigned char base64_2_bin[256] = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 62, 0, 0, 0, 63,
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 62, 0, 62, 0, 63,
 	52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 0, 0, 0, 0, 0, 0,
 	0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
-	15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 0, 0, 0, 0, 0,
+	15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 0, 0, 0, 0, 63,
 	0, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
 	41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 0, 0, 0, 0, 0,
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -61,10 +61,11 @@ static const unsigned char base64_2_bin[256] = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
-static int G_GNUC_WGET_CONST _isbase64(char c)
+static bool WGET_GCC_CONST isbase64(char c)
 {
 	// isalnum(c) does not work for all locales
-	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') || c == '+' || c == '/';
+	return base64_2_bin[(unsigned char) c] != 0;
+//	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') || c == '+' || c == '/' || c == '-' || c == '_';
 }
 
 /**
@@ -77,7 +78,7 @@ static int G_GNUC_WGET_CONST _isbase64(char c)
 bool wget_base64_is_string(const char *src)
 {
 	if (src) {
-		while (_isbase64(*src)) src++;
+		while (isbase64(*src)) src++;
 
 		if (!*src || (*src == '=' && src[1]) || (*src == '=' && src[1] == '=' && src[2]))
 			return 1;
@@ -104,7 +105,7 @@ size_t wget_base64_decode(char *dst, const char *src, size_t n)
 	int extra;
 
 	// trim '=' at the end
-	while (n > 0 && !_isbase64(usrc[n - 1]))
+	while (n > 0 && !isbase64(src[n - 1]))
 		n--;
 
 	extra = n & 3;
@@ -143,7 +144,7 @@ size_t wget_base64_decode(char *dst, const char *src, size_t n)
  * \param[in] src Base64 string to be decoded
  * \param[in] n Length of \p src
  * \param[out] outlen Length of returned string, may be NULL.
- * \return Decoded bytes, zero terminated
+ * \return Decoded bytes, zero terminated. Returns NULL if memory allocation failed.
  *
  * Decodes \p n bytes of the base64 string \p src.
  * The decoded bytes are returned in an allocated buffer.
@@ -152,7 +153,10 @@ size_t wget_base64_decode(char *dst, const char *src, size_t n)
  */
 char *wget_base64_decode_alloc(const char *src, size_t n, size_t *outlen)
 {
-	char *dst = xmalloc(((n + 3) / 4) * 3 + 1);
+	char *dst = wget_malloc(wget_base64_get_decoded_length(n));
+
+	if (!dst)
+		return NULL;
 
 	size_t _outlen = wget_base64_decode(dst, src, n);
 
@@ -162,22 +166,16 @@ char *wget_base64_decode_alloc(const char *src, size_t n, size_t *outlen)
 	return dst;
 }
 
-/**
- * \param[out] dst Base64 output string
- * \param[in] src Input buffer
- * \param[in] n Number of bytes to be encoded
- * \return Length of output string \p dst
- *
- * Encodes \p n bytes from \p src into a base64 string.
- * The encoded string is written into \p dst (0-terminated).
- *
- * The length of \p dst has to be at minimum ((\p n + 2) / 3) * 4 + 1 bytes.
- */
-size_t wget_base64_encode(char *dst, const char *src, size_t n)
-{
-	static const char base64[64] =
-		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+#define WGET_BASE64_URLENCODE 1
 
+static size_t base64_encode(char *dst, const char *src, size_t n, int flags)
+{
+	static const char base64unsafe[64] =
+		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+	static const char base64urlsafe[64] =
+		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
+
+	const char *base64 = (flags & WGET_BASE64_URLENCODE) ? base64urlsafe : base64unsafe;
 	const unsigned char *usrc = (const unsigned char *)src;
 	char *start = dst;
 	int extra = n % 3;
@@ -209,9 +207,41 @@ size_t wget_base64_encode(char *dst, const char *src, size_t n)
 }
 
 /**
+ * \param[out] dst Base64 output string
  * \param[in] src Input buffer
  * \param[in] n Number of bytes to be encoded
- * \return Base64 encoded string
+ * \return Length of output string \p dst
+ *
+ * Encodes \p n bytes from \p src into a base64 string.
+ * The encoded string is written into \p dst (0-terminated).
+ *
+ * The length of \p dst has to be at minimum ((\p n + 2) / 3) * 4 + 1 bytes.
+ */
+size_t wget_base64_encode(char *dst, const char *src, size_t n)
+{
+	return base64_encode(dst, src, n, 0);
+}
+
+/**
+ * \param[out] dst Base64 output string (URL and filename safe)
+ * \param[in] src Input buffer
+ * \param[in] n Number of bytes to be encoded
+ * \return Length of output string \p dst
+ *
+ * Encodes \p n bytes from \p src into a base64 URL and filename safe string (see RFC 4648, 5.).
+ * The encoded string is written into \p dst (0-terminated).
+ *
+ * The length of \p dst has to be at minimum ((\p n + 2) / 3) * 4 + 1 bytes.
+ */
+size_t wget_base64_urlencode(char *dst, const char *src, size_t n)
+{
+	return base64_encode(dst, src, n, WGET_BASE64_URLENCODE);
+}
+
+/**
+ * \param[in] src Input buffer
+ * \param[in] n Number of bytes to be encoded
+ * \return Base64 encoded string or NULL if memory allocation failed
  *
  * Encodes \p n bytes from input buffer \p src.
  * The encoded string is returned in an allocated buffer.
@@ -220,9 +250,10 @@ size_t wget_base64_encode(char *dst, const char *src, size_t n)
  */
 char *wget_base64_encode_alloc(const char *src, size_t n)
 {
-	char *dst = xmalloc(((n + 2) / 3) * 4 + 1);
+	char *dst = wget_malloc(wget_base64_get_encoded_length(n));
 
-	wget_base64_encode(dst, src, n);
+	if (dst)
+		wget_base64_encode(dst, src, n);
 
 	return dst;
 }
@@ -275,4 +306,29 @@ char *wget_base64_encode_printf_alloc(const char *fmt, ...)
 	return dst;
 }
 
+/**
+ * \fn static inline size_t wget_base64_get_decoded_length(size_t len)
+ * \param[in] len Length of base64 sequence
+ * \return Number of decoded bytes plus one (for 0-byte termination)
+ *
+ * Calculate the number of bytes needed for decoding a base64 sequence with length \p len.
+ */
+size_t wget_base64_get_decoded_length(size_t len)
+{
+	return ((len + 3) / 4) * 3 + 1;
+}
+
+/**
+ * \fn static inline size_t wget_base64_get_encoded_length(size_t len)
+ * \param[in] len Number of (un-encoded) bytes
+ * \return Length of base64 encoding plus one (for 0-byte termination)
+ *
+ * Calculate the number of bytes needed for base64 encoding a byte sequence with length \p len,
+ * including the padding and 0-termination bytes.
+ */
+size_t wget_base64_get_encoded_length(size_t len)
+{
+	return ((len + 2) / 3) * 4 + 1;
+}
+
 /**@}*/
