@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -203,7 +203,7 @@ wget_buffer *wget_buffer_alloc(size_t size)
 	return buf;
 }
 
-static int _buffer_realloc(wget_buffer *buf, size_t size)
+static int buffer_realloc(wget_buffer *buf, size_t size)
 {
 	char *old_data = buf->data;
 
@@ -246,7 +246,7 @@ int wget_buffer_ensure_capacity(wget_buffer *buf, size_t size)
 {
 	if (likely(buf)) {
 		if (buf->size < size)
-			return _buffer_realloc(buf, size);
+			return buffer_realloc(buf, size);
 	}
 
 	return WGET_E_SUCCESS;
@@ -382,7 +382,7 @@ size_t wget_buffer_memcat(wget_buffer *buf, const void *data, size_t length)
 
 	if (likely(length)) {
 		if (buf->size < buf->length + length)
-			if (_buffer_realloc(buf, buf->size * 2 + length) != WGET_E_SUCCESS)
+			if (buffer_realloc(buf, buf->size * 2 + length) != WGET_E_SUCCESS)
 				return buf->length;
 
 		if (likely(data))
@@ -519,7 +519,7 @@ size_t wget_buffer_memset_append(wget_buffer *buf, char c, size_t length)
 
 	if (likely(length)) {
 		if (unlikely(buf->size < buf->length + length))
-			if (_buffer_realloc(buf, buf->size * 2 + length) != WGET_E_SUCCESS)
+			if (buffer_realloc(buf, buf->size * 2 + length) != WGET_E_SUCCESS)
 				return buf->length;
 
 		memset(buf->data + buf->length, c, length);
@@ -544,26 +544,19 @@ char *wget_buffer_trim(wget_buffer *buf)
 	if (unlikely(!buf))
 		return NULL;
 
-	if (buf->length) {
-		char *start = buf->data;
-		char *end = start + buf->length - 1;
+	while (buf->length > 0 && isspace(buf->data[buf->length - 1])) {
+		buf->length--;
+	}
+	buf->data[buf->length] = 0;
 
-		if (isspace(*end)) {
-			/* Skip trailing spaces */
-			for (; isspace(*end) && end >= start; end--)
-				;
-			end[1] = 0;
-			buf->length = (size_t) (end - start + 1);
-		}
+	size_t spaces = 0;
+	while (buf->length > 0 && isspace(buf->data[spaces]))
+		spaces++;
 
-		if (isspace(*start)) {
-			/* Skip leading spaces */
-			for (; isspace(*start) && end >= start; start++)
-				;
-			buf->length = (size_t) (end - start + 1);
-			/* Include trailing 0 */
-			memmove(buf->data, start, buf->length + 1);
-		}
+	if (spaces) {
+		buf->length -= spaces;
+		/* Include trailing 0 */
+		memmove(buf->data, buf->data + spaces, buf->length + 1);
 	}
 
 	return buf->data;
