@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2012 Tim Ruehsen
- * Copyright (c) 2015-2022 Free Software Foundation, Inc.
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdbool.h>
 #include <c-ctype.h>
 
 #include <wget.h>
@@ -62,14 +63,15 @@ static void copy_string(wget_buffer *buf, unsigned int flags, int field_width, i
 		return;
 	}
 
-	length = strlen(arg);
+	if (precision >= 0) {
+		length = strnlen(arg, precision);
+	} else {
+		length = strlen(arg);
+	}
 
-	// info_printf("flags=0x%02x field_width=%d precision=%d length=%zd arg='%s'\n",
+	// debug_printf("flags=0x%02x field_width=%d precision=%d length=%zu arg='%s'\n",
 	//	flags,field_width,precision,length,arg);
 
-	if (precision >= 0 && length > (size_t)precision)
-		length = precision;
-
 	if (field_width) {
 		if ((unsigned)field_width > length) {
 			if (flags & FLAG_LEFT_ADJUST) {
@@ -259,9 +261,9 @@ static void convert_pointer(wget_buffer *buf, void *pointer)
 	wget_buffer_memcat(buf, dst, length);
 }
 
-static const char *read_precision(const char *p, int *out, int precision_is_external)
+static const char *read_precision(const char *p, int *out, bool precision_is_external)
 {
-	int precision = -1;
+	int precision;
 
 	if (precision_is_external) {
 		precision = *out;
@@ -300,7 +302,7 @@ static const char *read_flag_chars(const char *p, unsigned int *out)
 	return p;
 }
 
-static const char *read_field_width(const char *p, int *out, unsigned int *flags, int width_is_external)
+static const char *read_field_width(const char *p, int *out, unsigned int *flags, bool width_is_external)
 {
 	int field_width;
 
