@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -37,50 +37,64 @@
 typedef struct {
 	const char
 		**encoding;
-	wget_vector_t
+	wget_vector
 		*uris;
-} _CSS_CONTEXT;
+} css_context;
 
-static void _free_url(wget_css_parsed_url_t *url)
+static void url_free(void *url)
 {
-	xfree(url->url);
-	xfree(url->abs_url);
+	wget_css_parsed_url *u = url;
+
+	xfree(u->url);
+	xfree(u->abs_url);
+	xfree(u);
 }
 
 // Callback function, called from CSS parser for each @charset found.
-static void _css_get_encoding(void *context, const char *encoding, size_t len)
+static void get_encoding(void *context, const char *encoding, size_t len)
 {
-	_CSS_CONTEXT *ctx = context;
+	css_context *ctx = context;
 
 	// take only the first @charset rule
 	if (!*ctx->encoding) {
 		*ctx->encoding = wget_strmemdup(encoding, len);
-		debug_printf(_("URI content encoding = '%s'\n"), *ctx->encoding);
+		debug_printf("URI content encoding = '%s'\n", *ctx->encoding);
 	}
 }
 
 // Callback function, called from CSS parser for each URI found.
-static void _css_get_url(void *context, const char *url, size_t len, size_t pos)
+static void get_url(void *context, const char *url, size_t len, size_t pos)
 {
-	_CSS_CONTEXT *ctx = context;
-	wget_css_parsed_url_t parsed_url = { .len = len, .pos = pos, .url = wget_strmemdup(url, len), .abs_url = NULL };
+	css_context *ctx = context;
+	wget_css_parsed_url *parsed_url;
+
+	if (!(parsed_url = wget_calloc(1, sizeof(wget_css_parsed_url))))
+		return;
+
+	if (!(parsed_url->url = wget_strmemdup(url, len))) {
+		xfree(parsed_url);
+		return;
+	}
+
+	parsed_url->len = len;
+	parsed_url->pos = pos;
 
 	if (!ctx->uris) {
 		ctx->uris = wget_vector_create(16, NULL);
-		wget_vector_set_destructor(ctx->uris, (wget_vector_destructor_t)_free_url);
+		wget_vector_set_destructor(ctx->uris, url_free);
 	}
 
-	wget_vector_add(ctx->uris, &parsed_url, sizeof(parsed_url));
+	wget_vector_add(ctx->uris, parsed_url);
 }
 
-static void _urls_to_absolute(wget_vector_t *urls, wget_iri_t *base)
+static void urls_to_absolute(wget_vector *urls, wget_iri *base)
 {
 	if (base && urls) {
-		wget_buffer_t buf;
+		wget_buffer buf;
 		wget_buffer_init(&buf, NULL, 1024);
 
 		for (int it = 0; it < wget_vector_size(urls); it++) {
-			wget_css_parsed_url_t *url = wget_vector_get(urls, it);
+			wget_css_parsed_url *url = wget_vector_get(urls, it);
 
 			if (wget_iri_relative_to_abs(base, url->url, url->len, &buf))
 				url->abs_url = wget_strmemdup(buf.data, buf.length);
@@ -92,22 +106,22 @@ static void _urls_to_absolute(wget_vector_t *urls, wget_iri_t *base)
 	}
 }
 
-wget_vector_t *wget_css_get_urls(const char *css, size_t len, wget_iri_t *base, const char **encoding)
+wget_vector *wget_css_get_urls(const char *css, size_t len, wget_iri *base, const char **encoding)
 {
-	_CSS_CONTEXT context = { .encoding = encoding };
+	css_context context = { .encoding = encoding };
 
-	wget_css_parse_buffer(css, len, _css_get_url, encoding ? _css_get_encoding : NULL, &context);
-	_urls_to_absolute(context.uris, base);
+	wget_css_parse_buffer(css, len, get_url, encoding ? get_encoding : NULL, &context);
+	urls_to_absolute(context.uris, base);
 
 	return context.uris;
 }
 
-wget_vector_t *wget_css_get_urls_from_localfile(const char *fname, wget_iri_t *base, const char **encoding)
+wget_vector *wget_css_get_urls_from_localfile(const char *fname, wget_iri *base, const char **encoding)
 {
-	_CSS_CONTEXT context = { .encoding = encoding };
+	css_context context = { .encoding = encoding };
 
-	wget_css_parse_file(fname, _css_get_url, encoding ? _css_get_encoding : NULL, &context);
-	_urls_to_absolute(context.uris, base);
+	wget_css_parse_file(fname, get_url, encoding ? get_encoding : NULL, &context);
+	urls_to_absolute(context.uris, base);
 
 	return context.uris;
 }
