@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -53,9 +53,9 @@
  *
  * The ideal length of \p out would be wget_hash_get_len(type) * 2 + 1.
  */
-void wget_hash_printf_hex(wget_digest_algorithm_t algorithm, char *out, size_t outsize, const char *fmt, ...)
+void wget_hash_printf_hex(wget_digest_algorithm algorithm, char *out, size_t outsize, const char *fmt, ...)
 {
-	char *plaintext;
+	char *plaintext = NULL;
 	va_list args;
 	size_t len;
 
@@ -63,19 +63,33 @@ void wget_hash_printf_hex(wget_digest_algorithm_t algorithm, char *out, size_t o
 	len = wget_vasprintf(&plaintext, fmt, args);
 	va_end(args);
 
-	if (plaintext) {
-		unsigned char digest[wget_hash_get_len(algorithm)];
+	if (!plaintext)
+		return;
+
+	unsigned char tmp[256], *digest = tmp;
+	size_t digestlen = wget_hash_get_len(algorithm);
+
+	if (digestlen > sizeof(tmp)) {
+		digest = wget_malloc(digestlen);
+	}
+
+	if (digest) {
 		int rc;
 
 		if ((rc = wget_hash_fast(algorithm, plaintext, len, digest)) == 0) {
-			wget_memtohex(digest, sizeof(digest), out, outsize);
+			wget_memtohex(digest, digestlen, out, outsize);
 		} else {
 			*out = 0;
 			error_printf(_("Failed to hash (%d)\n"), rc);
 		}
 
-		xfree(plaintext);
+		if (digest != tmp)
+			xfree(digest);
+	} else {
+		error_printf(_("%s: Failed to malloc %zu bytes\n"), __func__, digestlen);
 	}
+
+	xfree(plaintext);
 }
 
 /**@}*/
