@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -36,12 +36,14 @@
 #include "private.h"
 
 typedef struct {
-	wget_html_parsed_result_t
+	wget_html_parsed_result
 		result;
-	wget_vector_t *
+	wget_vector *
 		additional_tags;
-	wget_vector_t *
+	wget_vector *
 		ignore_tags;
+	wget_string
+		download;
 	int
 		uri_index;
 	size_t
@@ -54,7 +56,7 @@ typedef struct {
 		* html,
 		* css_attr,
 		* css_dir;
-} _html_context_t;
+} html_context;
 
 // see https://stackoverflow.com/questions/2725156/complete-list-of-html-tag-attributes-which-have-a-url-value
 static const char maybe[256] = {
@@ -86,29 +88,33 @@ static const char attrs[][12] = {
 	"usemap"
 };
 
-static void _css_parse_uri(void *context, const char *url G_GNUC_WGET_UNUSED, size_t len, size_t pos)
+static void css_parse_uri(void *context, const char *url WGET_GCC_UNUSED, size_t len, size_t pos)
 {
-	_html_context_t *ctx = context;
+	html_context *ctx = context;
+	wget_html_parsed_result *res = &ctx->result;
+	wget_html_parsed_url *parsed_url;
 
-	wget_html_parsed_result_t *res = &ctx->result;
+	if (!(parsed_url = wget_malloc(sizeof(wget_html_parsed_url))))
+		return;
+
+	parsed_url->link_inline = 1;
+	wget_strscpy(parsed_url->attr, ctx->css_attr, sizeof(parsed_url->attr));
+	wget_strscpy(parsed_url->tag, ctx->css_dir, sizeof(parsed_url->tag));
+	parsed_url->url.p = (const char *) (ctx->html + ctx->css_start_offset + pos);
+	parsed_url->url.len = len;
+	parsed_url->download.p = NULL;
+	parsed_url->download.len = 0;
 
 	if (!res->uris)
 		res->uris = wget_vector_create(32, NULL);
 
-	wget_html_parsed_url_t parsed_url;
-	parsed_url.link_inline = 1;
-	wget_strscpy(parsed_url.attr, ctx->css_attr, sizeof(parsed_url.attr));
-	wget_strscpy(parsed_url.dir, ctx->css_dir, sizeof(parsed_url.dir));
-	parsed_url.url.p = (const char *) (ctx->html + ctx->css_start_offset + pos);
-	parsed_url.url.len = len;
-
-	wget_vector_add(res->uris, &parsed_url, sizeof(parsed_url));
+	wget_vector_add(res->uris, parsed_url);
 }
 
 // Callback function, called from HTML parser for each URI found.
-static void _html_get_url(void *context, int flags, const char *tag, const char *attr, const char *val, size_t len, size_t pos G_GNUC_WGET_UNUSED)
+static void html_get_url(void *context, int flags, const char *tag, const char *attr, const char *val, size_t len, size_t pos WGET_GCC_UNUSED)
 {
-	_html_context_t *ctx = context;
+	html_context *ctx = context;
 
 	// Read the encoding from META tag, e.g. from
 	//   <meta http-equiv="Content-Type" content="text/html; charset=utf-8">.
@@ -117,8 +123,16 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 	// Also ,we are interested in ROBOTS e.g.
 	//   <META name="ROBOTS" content="NOINDEX, NOFOLLOW">
 	if ((flags & XML_FLG_BEGIN)) {
-		if ((*tag|0x20) == 'm' && !wget_strcasecmp_ascii(tag, "meta"))
+		if ((*tag|0x20) == 'a' && (tag[1] == 0 || !wget_strcasecmp_ascii(tag, "area"))) {
+			// The download attribute is only valid for 'a' and 'area' tags.
+			// S 4.6.5 in https://html.spec.whatwg.org/multipage/links.html#downloading-resources
+			ctx->uri_index = -1;
+			ctx->download.p = NULL;
+			ctx->download.len = 0;
+		}
+		else if ((*tag|0x20) == 'm' && !wget_strcasecmp_ascii(tag, "meta")) {
 			ctx->found_robots = ctx->found_content_type = 0;
+		}
 		else if ((*tag|0x20) == 'l' && !wget_strcasecmp_ascii(tag, "link")) {
 			ctx->link_inline = 0;
 			ctx->uri_index = -1;
@@ -126,7 +140,7 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 	}
 
 	if ((flags & XML_FLG_ATTRIBUTE) && val) {
-		wget_html_parsed_result_t *res = &ctx->result;
+		wget_html_parsed_result *res = &ctx->result;
 
 //		debug_printf("%02X %s %s '%.*s' %zu %zu\n", (unsigned) flags, tag, attr, (int) len, val, len, pos);
 
@@ -137,13 +151,15 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 					return;
 				}
 			} else if (ctx->found_robots && !wget_strcasecmp_ascii(attr, "content")) {
-				char *p;
-				char valbuf[len + 1], *value = valbuf;
+				char valbuf[256], *valp;
+				const char *value;
 
-				memcpy(value, val, len);
-				value[len] = 0;
+				if (!(value = valp = wget_strmemcpy_a(valbuf, sizeof(valbuf), val, len)))
+					return;
 
 				while (*value) {
+					const char *p;
+
 					while (c_isspace(*value)) value++;
 					if (*value == ',') { value++; continue; }
 					for (p = value; *p && !c_isspace(*p) && *p != ','; p++);
@@ -157,16 +173,25 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 
 					value = *p  ? p + 1 : p;
 				}
+
+				if (valp != valbuf)
+					xfree(valp);
+
 				return;
 			}
 
 			if (ctx->found_content_type && !res->encoding) {
 				if (!wget_strcasecmp_ascii(attr, "content")) {
-					char valbuf[len + 1], *value = valbuf;
+					char valbuf[256];
+					const char *value;
+
+					if (!(value = wget_strmemcpy_a(valbuf, sizeof(valbuf), val, len)))
+						return;
 
-					memcpy(value, val, len);
-					value[len] = 0;
 					wget_http_parse_content_type(value, NULL, &res->encoding);
+
+					if (value != valbuf)
+						xfree(value);
 				}
 			}
 			else if (!ctx->found_content_type && !res->encoding) {
@@ -182,8 +207,8 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 		}
 
 		if (ctx->ignore_tags) {
-			if (wget_vector_find(ctx->ignore_tags, &(wget_html_tag_t){ .name = tag, .attribute = NULL } ) != -1
-				|| wget_vector_find(ctx->ignore_tags, &(wget_html_tag_t){ .name = tag, .attribute = attr } ) != -1)
+			if (wget_vector_find(ctx->ignore_tags, &(wget_html_tag){ .name = tag, .attribute = NULL } ) != -1
+				|| wget_vector_find(ctx->ignore_tags, &(wget_html_tag){ .name = tag, .attribute = attr } ) != -1)
 				return;
 		}
 
@@ -191,27 +216,70 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 			ctx->css_dir = tag;
 			ctx->css_attr = "style";
 			ctx->css_start_offset = val - ctx->html;
-			wget_css_parse_buffer(val, len, _css_parse_uri, NULL, context);
+			wget_css_parse_buffer(val, len, css_parse_uri, NULL, context);
 			return;
 		}
 
 		if ((*tag|0x20) == 'l' && !wget_strcasecmp_ascii(tag, "link")) {
 			if (!wget_strcasecmp_ascii(attr, "rel")) {
-				if (!wget_strncasecmp_ascii(val, "shortcut icon", len)
-					|| !wget_strncasecmp_ascii(val, "stylesheet", len)
-					|| !wget_strncasecmp_ascii(val, "preload", len))
-					ctx->link_inline = 1;
-				else
-					ctx->link_inline = 0;
+				ctx->link_inline = 0;
+
+				// "rel" contains a space separated list of items.
+				//   see https://html.spec.whatwg.org/multipage/semantics.html#attr-link-rel
+				//   see https://html.spec.whatwg.org/multipage/links.html#linkTypes
+				while (len) {
+					const char *p;
+
+					for (p = val;len && !c_isspace(*val); val++, len--); // find end of item
+					if (p == val) { val++; len--; continue; } // found a delimiter
+
+					// Check for items that may be important to display the page.
+					if (!wget_strncasecmp_ascii(p, "icon", val - p)
+						|| !wget_strncasecmp_ascii(p, "manifest", val - p)
+						|| !wget_strncasecmp_ascii(p, "modulepreload", val - p)
+						|| !wget_strncasecmp_ascii(p, "stylesheet", val - p)
+						|| !wget_strncasecmp_ascii(p, "prefetch", val - p)
+						|| !wget_strncasecmp_ascii(p, "preload", val - p))
+					{
+						ctx->link_inline = 1;
+						break;
+					}
+				}
 
 				if (ctx->uri_index >= 0) {
 					// href= came before rel=
-					wget_html_parsed_url_t *url = wget_vector_get(res->uris, ctx->uri_index);
+					wget_html_parsed_url *url = wget_vector_get(res->uris, ctx->uri_index);
 					url->link_inline = ctx->link_inline;
 				}
+				return;
 			}
 		}
 
+		if ((*tag|0x20) == 'a' && (tag[1] == 0 || !wget_strcasecmp_ascii(tag, "area"))
+			&& !wget_strcasecmp_ascii(attr, "download"))
+		{
+			if (!val)
+				return;
+
+			for (;len && c_isspace(*val); val++, len--); // skip leading spaces
+			for (;len && c_isspace(val[len - 1]); len--);  // skip trailing spaces
+			if (!len)
+				return;
+
+			// remember for later
+			ctx->download.p = val;
+			ctx->download.len = len;
+
+			if (ctx->uri_index >= 0) {
+				// href= came before download=
+				wget_html_parsed_url *url = wget_vector_get(res->uris, ctx->uri_index);
+				url->download.p = val;
+				url->download.len = len;
+			}
+
+			return;
+		}
+
 		// shortcut to avoid unneeded calls to bsearch()
 		int found = 0;
 
@@ -221,8 +289,8 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 
 		// search the dynamic list for a tag/attr match
 		if (!found && ctx->additional_tags) {
-			if (wget_vector_find(ctx->additional_tags, &(wget_html_tag_t){ .name = tag, .attribute = NULL } ) != -1
-				|| wget_vector_find(ctx->additional_tags, &(wget_html_tag_t){ .name = tag, .attribute = attr } ) != -1)
+			if (wget_vector_find(ctx->additional_tags, &(wget_html_tag){ .name = tag, .attribute = NULL } ) != -1
+				|| wget_vector_find(ctx->additional_tags, &(wget_html_tag){ .name = tag, .attribute = attr } ) != -1)
 				found = 1;
 		}
 
@@ -240,21 +308,30 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 			if (!res->uris)
 				res->uris = wget_vector_create(32, NULL);
 
-			wget_html_parsed_url_t url;
+			wget_html_parsed_url url;
 
 			if (!wget_strcasecmp_ascii(attr, "srcset")) {
 				// value is a list of URLs, see https://html.spec.whatwg.org/multipage/embedded-content.html#attr-img-srcset
+				// See also https://html.spec.whatwg.org/multipage/images.html#srcset-attribute
 				while (len) {
 					const char *p;
 
 					for (;len && c_isspace(*val); val++, len--); // skip leading spaces
 					for (p = val;len && !c_isspace(*val) && *val != ','; val++, len--); // find end of URL
 					if (p != val) {
+						// The 'data:' URL contains a single comma: https://datatracker.ietf.org/doc/html/rfc2397
+						if (len && *val == ',' && !wget_strncasecmp_ascii(p, "data:", 5)) {
+							// advance to the end of the 'data:' URL
+							for (val++, len--;len && !c_isspace(*val) && *val != ','; val++, len--);
+						}
+						url.download.p = NULL;
+						url.download.len = 0;
+						url.link_inline = ctx->link_inline;
 						wget_strscpy(url.attr, attr, sizeof(url.attr));
-						wget_strscpy(url.dir, tag, sizeof(url.dir));
+						wget_strscpy(url.tag, tag, sizeof(url.tag));
 						url.url.p = p;
 						url.url.len = val - p;
-						wget_vector_add(res->uris, &url, sizeof(url));
+						wget_vector_add_memdup(res->uris, &url, sizeof(url));
 					}
 					for (;len && *val != ','; val++, len--); // skip optional width/density descriptor
 					if (len && *val == ',') { val++; len--; }
@@ -262,12 +339,14 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 
 			} else {
 				// value is a single URL
+				url.download.p = ctx->download.p;
+				url.download.len = ctx->download.len;
 				url.link_inline = ctx->link_inline;
 				wget_strscpy(url.attr, attr, sizeof(url.attr));
-				wget_strscpy(url.dir, tag, sizeof(url.dir));
+				wget_strscpy(url.tag, tag, sizeof(url.tag));
 				url.url.p = val;
 				url.url.len = len;
-				ctx->uri_index = wget_vector_add(res->uris, &url, sizeof(url));
+				ctx->uri_index = wget_vector_add_memdup(res->uris, &url, sizeof(url));
 			}
 		}
 	}
@@ -276,32 +355,11 @@ static void _html_get_url(void *context, int flags, const char *tag, const char
 		ctx->css_dir = "style";
 		ctx->css_attr = "";
 		ctx->css_start_offset = val - ctx->html;
-		wget_css_parse_buffer(val, len, _css_parse_uri, NULL, context);
+		wget_css_parse_buffer(val, len, css_parse_uri, NULL, context);
 	}
 }
 
-/*
-static void _urls_to_absolute(WGET_VECTOR *urls, WGET_IRI *base)
-{
-	if (base && urls) {
-		wget_buffer_t buf;
-		wget_buffer_init(&buf, NULL, 1024);
-
-		for (int it = 0; it < wget_vector_size(urls); it++) {
-			WGET_PARSED_URL *url = wget_vector_get(urls, it);
-
-			if (wget_iri_relative_to_abs(base, url->url, url->len, &buf))
-				url->abs_url = wget_strmemdup(buf.data, buf.length);
-			else
-				error_printf(_("Cannot resolve relative URI '%s'\n"), url->url);
-		}
-
-		wget_buffer_deinit(&buf);
-	}
-}
-*/
-
-void wget_html_free_urls_inline (wget_html_parsed_result_t **res)
+void wget_html_free_urls_inline (wget_html_parsed_result **res)
 {
 	if (res && *res) {
 		xfree((*res)->encoding);
@@ -310,9 +368,9 @@ void wget_html_free_urls_inline (wget_html_parsed_result_t **res)
 	}
 }
 
-wget_html_parsed_result_t *wget_html_get_urls_inline(const char *html, wget_vector_t *additional_tags, wget_vector_t *ignore_tags)
+wget_html_parsed_result *wget_html_get_urls_inline(const char *html, wget_vector *additional_tags, wget_vector *ignore_tags)
 {
-	_html_context_t context = {
+	html_context context = {
 		.result.follow = 1,
 		.additional_tags = additional_tags,
 		.ignore_tags = ignore_tags,
@@ -320,7 +378,7 @@ wget_html_parsed_result_t *wget_html_get_urls_inline(const char *html, wget_vect
 	};
 
 //	context.result.uris = wget_vector_create(32, -2, NULL);
-	wget_html_parse_buffer(html, _html_get_url, &context, HTML_HINT_REMOVE_EMPTY_CONTENT);
+	wget_html_parse_buffer(html, html_get_url, &context, HTML_HINT_REMOVE_EMPTY_CONTENT);
 
 	return wget_memdup(&context.result, sizeof(context.result));
 }
