@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2022 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -848,16 +848,13 @@ int64_t wget_http_parse_full_date(const char *s)
 	int day, mon = 0, year, hour, min, sec, leap_month, leap_year, days;
 	char mname[4] = "";
 
-	if (sscanf(s, " %*[a-zA-Z], %02d %3s %4d %2d:%2d:%2d", &day, mname, &year, &hour, &min, &sec) >= 6) {
+	if (sscanf(s, " %*[a-zA-Z], %2d %3s %4d %2d:%2d:%2d", &day, mname, &year, &hour, &min, &sec) == 6) {
 		// RFC 822 / 1123: Wed, 09 Jun 2021 10:18:14 GMT
-	}
-	else if (sscanf(s, " %*[a-zA-Z], %2d-%3s-%4d %2d:%2d:%2d", &day, mname, &year, &hour, &min, &sec) >= 6) {
+	} else if (sscanf(s, " %*[a-zA-Z], %2d-%3s-%4d %2d:%2d:%2d", &day, mname, &year, &hour, &min, &sec) == 6) {
 		// RFC 850 / 1036 or Netscape: Wednesday, 09-Jun-21 10:18:14 or Wed, 09-Jun-2021 10:18:14
-	}
-	else if (sscanf(s, " %*[a-zA-Z] %3s %2d %2d:%2d:%2d %4d", mname, &day, &hour, &min, &sec, &year) >= 6) {
+	} else if (sscanf(s, " %*[a-zA-Z] %3s %2d %2d:%2d:%2d %4d", mname, &day, &hour, &min, &sec, &year) == 6) {
 		// ANSI C's asctime(): Wed Jun 09 10:18:14 2021
-	}
-	else if (sscanf(s, " %d %3s %4d %2d:%2d:%2d", &day, mname, &year, &hour, &min, &sec) >= 6) {
+	} else if (sscanf(s, " %d %3s %4d %2d:%2d:%2d", &day, mname, &year, &hour, &min, &sec) == 6) {
 		// non-standard: 1 Mar 2027 09:23:12 GMT
 	} else {
 		error_printf(_("Failed to parse date '%s'\n"), s);
