@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2014 Tim Ruehsen
- * Copyright (c) 2015-2021 Free Software Foundation, Inc.
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -509,14 +509,15 @@ int wget_ocsp_db_load(wget_ocsp_db *ocsp_db)
 	if (!ocsp_db->fname || !*ocsp_db->fname)
 		return -1;
 
-	char fname_hosts[strlen(ocsp_db->fname) + 6 + 1];
-	wget_snprintf(fname_hosts, sizeof(fname_hosts), "%s_hosts", ocsp_db->fname);
+	char *fname_hosts = wget_aprintf("%s_hosts", ocsp_db->fname);
 
 	if ((ret = wget_update_file(fname_hosts, ocsp_db_load_hosts, NULL, ocsp_db)))
 		error_printf(_("Failed to read OCSP hosts\n"));
 	else
 		debug_printf("Fetched OCSP hosts from '%s'\n", fname_hosts);
 
+	xfree(fname_hosts);
+
 	if (wget_update_file(ocsp_db->fname, ocsp_db_load_fingerprints, NULL, ocsp_db)) {
 		error_printf(_("Failed to read OCSP fingerprints\n"));
 		ret = -1;
@@ -527,15 +528,21 @@ int wget_ocsp_db_load(wget_ocsp_db *ocsp_db)
 }
 
 WGET_GCC_NONNULL_ALL
-static int ocsp_save_fingerprint(FILE *fp, const ocsp_entry *ocsp)
+static int ocsp_save_fingerprint(void *_fp, const void *_ocsp, WGET_GCC_UNUSED void *v)
 {
+	FILE *fp = _fp;
+	const ocsp_entry *ocsp = _ocsp;
+
 	wget_fprintf(fp, "%s %lld %lld %d\n", ocsp->key, (long long)ocsp->maxage, (long long)ocsp->mtime, ocsp->valid);
 	return 0;
 }
 
 WGET_GCC_NONNULL_ALL
-static int ocsp_save_host(FILE *fp, const ocsp_entry *ocsp)
+static int ocsp_save_host(void *_fp, const void *_ocsp, WGET_GCC_UNUSED void *v)
 {
+	FILE *fp = _fp;
+	const ocsp_entry *ocsp = _ocsp;
+
 	wget_fprintf(fp, "%s %lld %lld\n", ocsp->key, (long long)ocsp->maxage, (long long)ocsp->mtime);
 	return 0;
 }
@@ -548,7 +555,7 @@ static int ocsp_db_save_hosts(void *ocsp_db, FILE *fp)
 		fputs("#OCSP 1.0 host file\n", fp);
 		fputs("#Generated by libwget " PACKAGE_VERSION ". Edit at your own risk.\n", fp);
 		fputs("<hostname> <time_t maxage> <time_t mtime>\n\n", fp);
-		wget_hashmap_browse(map, (wget_hashmap_browse_fn *) ocsp_save_host, fp);
+		wget_hashmap_browse(map, ocsp_save_host, fp);
 
 		if (ferror(fp))
 			return -1;
@@ -566,7 +573,7 @@ static int ocsp_db_save_fingerprints(void *ocsp_db, FILE *fp)
 		fputs("#OCSP 1.0 fingerprint file\n", fp);
 		fputs("#Generated by Wget " PACKAGE_VERSION ". Edit at your own risk.\n", fp);
 		fputs("<sha256 fingerprint of cert> <time_t maxage> <time_t mtime> <valid>\n\n", fp);
-		wget_hashmap_browse(map, (wget_hashmap_browse_fn *) ocsp_save_fingerprint, fp);
+		wget_hashmap_browse(map, ocsp_save_fingerprint, fp);
 
 		if (ferror(fp))
 			return -1;
@@ -596,14 +603,15 @@ int wget_ocsp_db_save(wget_ocsp_db *ocsp_db)
 	if (!ocsp_db || !ocsp_db->fname || !*ocsp_db->fname)
 		return -1;
 
-	char fname_hosts[strlen(ocsp_db->fname) + 6 + 1];
-	wget_snprintf(fname_hosts, sizeof(fname_hosts), "%s_hosts", ocsp_db->fname);
+	char *fname_hosts = wget_aprintf("%s_hosts", ocsp_db->fname);
 
 	if ((ret = wget_update_file(fname_hosts, ocsp_db_load_hosts, ocsp_db_save_hosts, ocsp_db)))
 		error_printf(_("Failed to write to OCSP hosts to '%s'\n"), fname_hosts);
 	else
 		debug_printf("Saved OCSP hosts to '%s'\n", fname_hosts);
 
+	xfree(fname_hosts);
+
 	if (wget_update_file(ocsp_db->fname, ocsp_db_load_fingerprints, ocsp_db_save_fingerprints, ocsp_db)) {
 		error_printf(_("Failed to write to OCSP fingerprints to '%s'\n"), ocsp_db->fname);
 		ret = -1;
