@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2019 Free Software Foundation, Inc.
+ * Copyright (c) 2019-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -38,12 +38,25 @@
 #include <string.h>
 #include <errno.h>
 
+#ifdef _WIN32
+#  include <w32sock.h>
+#else
+#  define FD_TO_SOCKET(x) (x)
+#  define SOCKET_TO_FD(x) (x)
+#endif
+
+#ifndef DEBUG_WOLFSSL
 #include <wolfssl/options.h>
+#undef DEBUG_WOLFSSL
+#else
+#include <wolfssl/options.h>
+#endif
 #include <wolfssl/ssl.h>
 
 #include <wget.h>
 #include "private.h"
 #include "net.h"
+#include "filename.h"
 
 /**
  * \file
@@ -63,7 +76,7 @@ static wget_ocsp_stats_callback
 static void
 	*ocsp_stats_ctx;
 
-static struct _config {
+static struct config {
 	const char
 		*secure_protocol,
 		*ca_directory,
@@ -86,12 +99,14 @@ static struct _config {
 		key_type;
 	bool
 		check_certificate : 1,
+		report_invalid_cert : 1,
 		check_hostname : 1,
 		print_info : 1,
 		ocsp : 1,
 		ocsp_stapling : 1;
-} _config = {
+} config = {
 	.check_certificate = 1,
+	.report_invalid_cert = 1,
 	.check_hostname = 1,
 	.ocsp = 1,
 	.ocsp_stapling = 1,
@@ -100,12 +115,13 @@ static struct _config {
 	.key_type = WGET_SSL_X509_FMT_PEM,
 	.secure_protocol = "AUTO",
 	.ca_directory = "system",
+	.ca_file = "system",
 #ifdef WITH_LIBNGHTTP2
 	.alpn = "h2,http/1.1",
 #endif
 };
 
-struct _session_context {
+struct session_context {
 	const char *
 		hostname;
 	wget_hpkp_stats_result
@@ -119,6 +135,8 @@ struct _session_context {
 static WOLFSSL_CTX
 	*ssl_ctx;
 
+#define error_printf_check(...) if (config.report_invalid_cert) wget_error_printf(__VA_ARGS__)
+
 /**
  * \param[in] key An identifier for the config parameter (starting with `WGET_SSL_`) to set
  * \param[in] value The value for the config parameter (a NULL-terminated string)
@@ -176,14 +194,14 @@ static WOLFSSL_CTX
 void wget_ssl_set_config_string(int key, const char *value)
 {
 	switch (key) {
-	case WGET_SSL_SECURE_PROTOCOL: _config.secure_protocol = value; break;
-	case WGET_SSL_CA_DIRECTORY: _config.ca_directory = value; break;
-	case WGET_SSL_CA_FILE: _config.ca_file = value; break;
-	case WGET_SSL_CERT_FILE: _config.cert_file = value; break;
-	case WGET_SSL_KEY_FILE: _config.key_file = value; break;
-	case WGET_SSL_CRL_FILE: _config.crl_file = value; break;
-	case WGET_SSL_OCSP_SERVER: _config.ocsp_server = value; break;
-	case WGET_SSL_ALPN: _config.alpn = value; break;
+	case WGET_SSL_SECURE_PROTOCOL: config.secure_protocol = value; break;
+	case WGET_SSL_CA_DIRECTORY: config.ca_directory = value; break;
+	case WGET_SSL_CA_FILE: config.ca_file = value; break;
+	case WGET_SSL_CERT_FILE: config.cert_file = value; break;
+	case WGET_SSL_KEY_FILE: config.key_file = value; break;
+	case WGET_SSL_CRL_FILE: config.crl_file = value; break;
+	case WGET_SSL_OCSP_SERVER: config.ocsp_server = value; break;
+	case WGET_SSL_ALPN: config.alpn = value; break;
 	default: error_printf(_("Unknown config key %d (or value must not be a string)\n"), key);
 	}
 }
@@ -215,9 +233,9 @@ void wget_ssl_set_config_string(int key, const char *value)
 void wget_ssl_set_config_object(int key, void *value)
 {
 	switch (key) {
-	case WGET_SSL_OCSP_CACHE: _config.ocsp_cert_cache = (wget_ocsp_db *)value; break;
-	case WGET_SSL_SESSION_CACHE: _config.tls_session_cache = (wget_tls_session_db *)value; break;
-	case WGET_SSL_HPKP_CACHE: _config.hpkp_cache = (wget_hpkp_db *)value; break;
+	case WGET_SSL_OCSP_CACHE: config.ocsp_cert_cache = (wget_ocsp_db *)value; break;
+	case WGET_SSL_SESSION_CACHE: config.tls_session_cache = (wget_tls_session_db *)value; break;
+	case WGET_SSL_HPKP_CACHE: config.hpkp_cache = (wget_hpkp_db *)value; break;
 	default: error_printf(_("Unknown config key %d (or value must not be an object)\n"), key);
 	}
 }
@@ -231,6 +249,7 @@ void wget_ssl_set_config_object(int key, void *value)
  * These are the parameters that can be set (\p key can have any of these values):
  *
  *  - WGET_SSL_CHECK_CERTIFICATE: whether certificates should be verified (1) or not (0)
+ *  - WGET_SSL_REPORT_INVALID_CERT: whether to print (1) errors/warnings regarding certificate validation or not (0)
  *  - WGET_SSL_CHECK_HOSTNAME: whether or not to check if the certificate's subject field
  *  matches the peer's hostname. This check is done according to the rules in [RFC 6125](https://tools.ietf.org/html/rfc6125)
  *  and typically involves checking whether the hostname and the common name (CN) field of the subject match.
@@ -256,14 +275,15 @@ void wget_ssl_set_config_object(int key, void *value)
 void wget_ssl_set_config_int(int key, int value)
 {
 	switch (key) {
-	case WGET_SSL_CHECK_CERTIFICATE: _config.check_certificate = (char)value; break;
-	case WGET_SSL_CHECK_HOSTNAME: _config.check_hostname = (char)value; break;
-	case WGET_SSL_CA_TYPE: _config.ca_type = (char)value; break;
-	case WGET_SSL_CERT_TYPE: _config.cert_type = (char)value; break;
-	case WGET_SSL_KEY_TYPE: _config.key_type = (char)value; break;
-	case WGET_SSL_PRINT_INFO: _config.print_info = (char)value; break;
-	case WGET_SSL_OCSP: _config.ocsp = (char)value; break;
-	case WGET_SSL_OCSP_STAPLING: _config.ocsp_stapling = (char)value; break;
+	case WGET_SSL_CHECK_CERTIFICATE: config.check_certificate = (char)value; break;
+	case WGET_SSL_REPORT_INVALID_CERT: config.report_invalid_cert = (char)value; break;
+	case WGET_SSL_CHECK_HOSTNAME: config.check_hostname = (char)value; break;
+	case WGET_SSL_CA_TYPE: config.ca_type = (char)value; break;
+	case WGET_SSL_CERT_TYPE: config.cert_type = (char)value; break;
+	case WGET_SSL_KEY_TYPE: config.key_type = (char)value; break;
+	case WGET_SSL_PRINT_INFO: config.print_info = (char)value; break;
+	case WGET_SSL_OCSP: config.ocsp = (char)value; break;
+	case WGET_SSL_OCSP_STAPLING: config.ocsp_stapling = (char)value; break;
 	default: error_printf(_("Unknown config key %d (or value must not be an integer)\n"), key);
 	}
 }
@@ -272,7 +292,7 @@ void wget_ssl_set_config_int(int key, int value)
  * if the hostname matches, as well as the activation, expiration dates.
  */
 /*
-static int _verify_certificate_callback(gnutls_session_t session)
+static int verify_certificate_callback(gnutls_session_t session)
 {
 	unsigned int status, deinit_cert = 0, deinit_issuer = 0;
 	const gnutls_datum_t *cert_list = 0;
@@ -280,11 +300,11 @@ static int _verify_certificate_callback(gnutls_session_t session)
 	int ret = -1, err, ocsp_ok = 0, pinning_ok = 0;
 	gnutls_x509_crt_t cert = NULL, issuer = NULL;
 	const char *hostname;
-	const char *tag = _config.check_certificate ? _("ERROR") : _("WARNING");
+	const char *tag = config.check_certificate ? _("ERROR") : _("WARNING");
 	unsigned nvalid = 0, nrevoked = 0, nignored = 0;
 
 	// read hostname
-	struct _session_context *ctx = gnutls_session_get_ptr(session);
+	struct session_context *ctx = gnutls_session_get_ptr(session);
 	hostname = ctx->hostname;
 
 	// This verification function uses the trusted CAs in the credentials
@@ -292,22 +312,22 @@ static int _verify_certificate_callback(gnutls_session_t session)
 	//
 	if (gnutls_certificate_verify_peers3(session, hostname, &status) != GNUTLS_E_SUCCESS) {
 //		if (wget_get_logger(WGET_LOGGER_DEBUG))
-//			_print_info(session);
-		error_printf(_("%s: Certificate verification error\n"), tag);
+//			print_info(session);
+		error_printf_check(_("%s: Certificate verification error\n"), tag);
 		goto out;
 	}
 
 //	if (wget_get_logger(WGET_LOGGER_DEBUG))
-//		_print_info(session);
+//		print_info(session);
 
 	if (status & GNUTLS_CERT_REVOKED) {
-		if (_config.ocsp_cert_cache)
-			wget_ocsp_db_add_host(_config.ocsp_cert_cache, hostname, 0); // remove entry from cache
+		if (config.ocsp_cert_cache)
+			wget_ocsp_db_add_host(config.ocsp_cert_cache, hostname, 0); // remove entry from cache
 		if (ctx->ocsp_stapling) {
 			if (gnutls_x509_crt_init(&cert) == GNUTLS_E_SUCCESS) {
 				if ((cert_list = gnutls_certificate_get_peers(session, &cert_list_size))) {
 					if (gnutls_x509_crt_import(cert, &cert_list[0], GNUTLS_X509_FMT_DER) == GNUTLS_E_SUCCESS) {
-						_add_cert_to_ocsp_cache(cert, 0);
+						add_cert_to_ocsp_cache(cert, 0);
 					}
 				}
 				gnutls_x509_crt_deinit(cert);
@@ -321,7 +341,7 @@ static int _verify_certificate_callback(gnutls_session_t session)
 		if (gnutls_certificate_verification_status_print(
 			status, gnutls_certificate_type_get(session), &out, 0) == GNUTLS_E_SUCCESS)
 		{
-			error_printf("%s: %s\n", tag, out.data); // no translation
+			error_printf_check("%s: %s\n", tag, out.data); // no translation
 			gnutls_free(out.data);
 		}
 
@@ -333,45 +353,45 @@ static int _verify_certificate_callback(gnutls_session_t session)
 	// be easily extended to work with openpgp keys as well.
 	//
 	if (gnutls_certificate_type_get(session) != GNUTLS_CRT_X509) {
-		error_printf(_("%s: Certificate must be X.509\n"), tag);
+		error_printf_check(_("%s: Certificate must be X.509\n"), tag);
 		goto out;
 	}
 
 	if (gnutls_x509_crt_init(&cert) != GNUTLS_E_SUCCESS) {
-		error_printf(_("%s: Error initializing X.509 certificate\n"), tag);
+		error_printf_check(_("%s: Error initializing X.509 certificate\n"), tag);
 		goto out;
 	}
 	deinit_cert = 1;
 
 	if (!(cert_list = gnutls_certificate_get_peers(session, &cert_list_size))) {
-		error_printf(_("%s: No certificate was found!\n"), tag);
+		error_printf_check(_("%s: No certificate was found!\n"), tag);
 		goto out;
 	}
 
 	if ((err = gnutls_x509_crt_import(cert, &cert_list[0], GNUTLS_X509_FMT_DER)) != GNUTLS_E_SUCCESS) {
-		error_printf(_("%s: Failed to parse certificate: %s\n"), tag, gnutls_strerror (err));
+		error_printf_check(_("%s: Failed to parse certificate: %s\n"), tag, gnutls_strerror (err));
 		goto out;
 	}
 
-	if (!_config.check_hostname || (_config.check_hostname && hostname && gnutls_x509_crt_check_hostname(cert, hostname)))
+	if (!config.check_hostname || (config.check_hostname && hostname && gnutls_x509_crt_check_hostname(cert, hostname)))
 		ret = 0;
 	else
 		goto out;
 
 	// At this point, the cert chain has been found valid regarding the locally available CA certificates and CRLs.
 	// Now, we are going to check the revocation status via OCSP
-	if (_config.ocsp_stapling) {
+	if (config.ocsp_stapling) {
 		if (!ctx->valid && ctx->ocsp_stapling) {
 			if (gnutls_ocsp_status_request_is_checked(session, 0)) {
 				debug_printf("Server certificate is valid regarding OCSP stapling\n");
-//				_get_cert_fingerprint(cert, fingerprint, sizeof(fingerprint)); // calc hexadecimal fingerprint string
-				_add_cert_to_ocsp_cache(cert, 1);
+//				get_cert_fingerprint(cert, fingerprint, sizeof(fingerprint)); // calc hexadecimal fingerprint string
+				add_cert_to_ocsp_cache(cert, 1);
 				nvalid = 1;
 			}
 			else if (gnutls_ocsp_status_request_is_checked(session, GNUTLS_OCSP_SR_IS_AVAIL))
-				error_printf(_("WARNING: The certificate's (stapled) OCSP status is invalid\n"));
-			else if (!_config.ocsp)
-				error_printf(_("WARNING: The certificate's (stapled) OCSP status has not been sent\n"));
+				error_printf_check(_("WARNING: The certificate's (stapled) OCSP status is invalid\n"));
+			else if (!config.ocsp)
+				error_printf_check(_("WARNING: The certificate's (stapled) OCSP status has not been sent\n"));
 		} else if (ctx->valid)
 			debug_printf("OCSP: Host '%s' is valid (from cache)\n", hostname);
 	}
@@ -381,22 +401,22 @@ static int _verify_certificate_callback(gnutls_session_t session)
 		gnutls_x509_crt_init(&cert);
 
 		if ((err = gnutls_x509_crt_import(cert, &cert_list[it], GNUTLS_X509_FMT_DER)) != GNUTLS_E_SUCCESS) {
-			error_printf(_("%s: Failed to parse certificate[%u]: %s\n"), tag, it, gnutls_strerror (err));
+			error_printf_check(_("%s: Failed to parse certificate[%u]: %s\n"), tag, it, gnutls_strerror (err));
 			continue;
 		}
 
-		if (_cert_verify_hpkp(cert, hostname, session) == 0)
+		if (cert_verify_hpkp(cert, hostname, session) == 0)
 			pinning_ok = 1;
 
-		_cert_verify_hpkp(cert, hostname, session);
+		cert_verify_hpkp(cert, hostname, session);
 
-		if (_config.ocsp && it > nvalid) {
+		if (config.ocsp && it > nvalid) {
 			char fingerprint[64 * 2 +1];
 			int revoked;
 
-			_get_cert_fingerprint(cert, fingerprint, sizeof(fingerprint)); // calc hexadecimal fingerprint string
+			get_cert_fingerprint(cert, fingerprint, sizeof(fingerprint)); // calc hexadecimal fingerprint string
 
-			if (wget_ocsp_fingerprint_in_cache(_config.ocsp_cert_cache, fingerprint, &revoked)) {
+			if (wget_ocsp_fingerprint_in_cache(config.ocsp_cert_cache, fingerprint, &revoked)) {
 				// found cert's fingerprint in cache
 				if (revoked) {
 					debug_printf("Certificate[%u] of '%s' has been revoked (cached)\n", it, hostname);
@@ -412,7 +432,7 @@ static int _verify_certificate_callback(gnutls_session_t session)
 				gnutls_x509_crt_deinit(issuer);
 				deinit_issuer = 0;
 			}
-			if ((err = gnutls_certificate_get_issuer(_credentials, cert, &issuer, 0)) != GNUTLS_E_SUCCESS && it < cert_list_size - 1) {
+			if ((err = gnutls_certificate_get_issuer(credentials, cert, &issuer, 0)) != GNUTLS_E_SUCCESS && it < cert_list_size - 1) {
 				gnutls_x509_crt_init(&issuer);
 				deinit_issuer = 1;
 				if ((err = gnutls_x509_crt_import(issuer, &cert_list[it + 1], GNUTLS_X509_FMT_DER))  != GNUTLS_E_SUCCESS) {
@@ -429,11 +449,11 @@ static int _verify_certificate_callback(gnutls_session_t session)
 
 			if (ocsp_ok == 1) {
 				debug_printf("Certificate[%u] of '%s' is valid (via OCSP)\n", it, hostname);
-				wget_ocsp_db_add_fingerprint(_config.ocsp_cert_cache, fingerprint, time(NULL) + 3600, 1); // 1h valid
+				wget_ocsp_db_add_fingerprint(config.ocsp_cert_cache, fingerprint, time(NULL) + 3600, 1); // 1h valid
 				nvalid++;
 			} else if (ocsp_ok == 0) {
 				debug_printf("%s: Certificate[%u] of '%s' has been revoked (via OCSP)\n", tag, it, hostname);
-				wget_ocsp_db_add_fingerprint(_config.ocsp_cert_cache, fingerprint, time(NULL) + 3600, 0);  // cert has been revoked
+				wget_ocsp_db_add_fingerprint(config.ocsp_cert_cache, fingerprint, time(NULL) + 3600, 0);  // cert has been revoked
 				nrevoked++;
 			} else {
 				debug_printf("WARNING: OCSP response not available or ignored\n");
@@ -442,7 +462,7 @@ static int _verify_certificate_callback(gnutls_session_t session)
 		}
 	}
 
-	if (_config.ocsp && stats_callback_ocsp) {
+	if (config.ocsp && stats_callback_ocsp) {
 		wget_ocsp_stats_data stats;
 		stats.hostname = hostname;
 		stats.nvalid = nvalid;
@@ -453,17 +473,17 @@ static int _verify_certificate_callback(gnutls_session_t session)
 		stats_callback_ocsp(&stats);
 	}
 
-	if (_config.ocsp_stapling || _config.ocsp) {
+	if (config.ocsp_stapling || config.ocsp) {
 		if (nvalid == cert_list_size) {
-			wget_ocsp_db_add_host(_config.ocsp_cert_cache, hostname, time(NULL) + 3600); // 1h valid
+			wget_ocsp_db_add_host(config.ocsp_cert_cache, hostname, time(NULL) + 3600); // 1h valid
 		} else if (nrevoked) {
-			wget_ocsp_db_add_host(_config.ocsp_cert_cache, hostname, 0); // remove entry from cache
+			wget_ocsp_db_add_host(config.ocsp_cert_cache, hostname, 0); // remove entry from cache
 			ret = -1;
 		}
 	}
 
 	if (!pinning_ok) {
-		error_printf(_("%s: Pubkey pinning mismatch!\n"), tag);
+		error_printf_check(_("%s: Pubkey pinning mismatch!\n"), tag);
 		ret = -1;
 	}
 
@@ -475,52 +495,63 @@ out:
 	if (deinit_issuer)
 		gnutls_x509_crt_deinit(issuer);
 
-	return _config.check_certificate ? ret : 0;
+	return config.check_certificate ? ret : 0;
 }
 */
 
-static int _init;
-static wget_thread_mutex _mutex;
+static int init;
+static wget_thread_mutex mutex;
 
-static void __attribute__ ((constructor)) _wget_tls_init(void)
+static void tls_exit(void)
 {
-	if (!_mutex)
-		wget_thread_mutex_init(&_mutex);
+	if (mutex)
+		wget_thread_mutex_destroy(&mutex);
 }
 
-static void __attribute__ ((destructor)) _wget_tls_exit(void)
+INITIALIZER(tls_init)
 {
-	if (_mutex)
-		wget_thread_mutex_destroy(&_mutex);
+	if (!mutex) {
+		wget_thread_mutex_init(&mutex);
+#ifdef DEBUG_WOLFSSL
+		wolfSSL_Debugging_ON();
+#endif // DEBUG_WOLFSSL
+
+		// Initialize paths while in a thread-safe environment (mostly for _WIN32).
+		wget_ssl_default_cert_dir();
+		wget_ssl_default_ca_bundle_path();
+
+		atexit(tls_exit);
+	}
 }
 
+
 /*
-static void _set_credentials(gnutls_certificate_credentials_t *credentials)
+static void set_credentials(gnutls_certificate_credentials_t *credentials)
 {
-	if (_config.cert_file && !_config.key_file) {
+	if (config.cert_file && !config.key_file) {
 		// Use the private key from the cert file unless otherwise specified.
-		_config.key_file = _config.cert_file;
-		_config.key_type = _config.cert_type;
+		config.key_file = config.cert_file;
+		config.key_type = config.cert_type;
 	}
-	else if (!_config.cert_file && _config.key_file) {
+	else if (!config.cert_file && config.key_file) {
 		// Use the cert from the private key file unless otherwise specified.
-		_config.cert_file = _config.key_file;
-		_config.cert_type = _config.key_type;
+		config.cert_file = config.key_file;
+		config.cert_type = config.key_type;
 	}
 
-	if (_config.cert_file && _config.key_file) {
-		if (_config.key_type != _config.cert_type) {
+	if (config.cert_file && config.key_file) {
+		if (config.key_type !=config.cert_type) {
 			// GnuTLS can't handle this
 			error_printf(_("GnuTLS requires the key and the cert to be of the same type.\n"));
 		}
 
-		if (gnutls_certificate_set_x509_key_file(*credentials, _config.cert_file, _config.key_file, _key_type(_config.key_type)) != GNUTLS_E_SUCCESS)
+		if (gnutls_certificate_set_x509_key_file(*credentials,config.cert_file,config.key_file,key_type(config.key_type)) != GNUTLS_E_SUCCESS)
 			error_printf(_("No certificates or keys were found\n"));
 	}
 
-	if (_config.ca_file) {
-		if (gnutls_certificate_set_x509_trust_file(*credentials, _config.ca_file, _key_type(_config.ca_type)) <= 0)
-			error_printf(_("No CAs were found in '%s'\n"), _config.ca_file);
+	if (config.ca_file) {
+		if (gnutls_certificate_set_x509_trust_file(*credentials, config.ca_file, key_type(config.ca_type)) <= 0)
+			error_printf(_("No CAs were found in '%s'\n"), config.ca_file);
 	}
 }
 */
@@ -547,45 +578,50 @@ static void _set_credentials(gnutls_certificate_credentials_t *credentials)
  */
 void wget_ssl_init(void)
 {
-	_wget_tls_init();
+	tls_init();
 
-	wget_thread_mutex_lock(_mutex);
+	wget_thread_mutex_lock(mutex);
 
-	if (!_init) {
+	if (!init) {
 		WOLFSSL_METHOD *method;
 		int min_version = -1;
 		const char *ciphers = NULL;
 
+#ifdef DEBUG_WOLFSSL
+		if (!wget_logger_is_active(wget_get_logger(WGET_LOGGER_DEBUG)))
+			wolfSSL_Debugging_OFF();
+#endif // DEBUG_WOLFSSL
+
 		debug_printf("WolfSSL init\n");
 		wolfSSL_Init();
 
-		if (!wget_strcasecmp_ascii(_config.secure_protocol, "SSLv2")) {
+		if (!wget_strcasecmp_ascii(config.secure_protocol, "SSLv2")) {
 			method = SSLv2_client_method();
-		} else if (!wget_strcasecmp_ascii(_config.secure_protocol, "SSLv3")) {
+		} else if (!wget_strcasecmp_ascii(config.secure_protocol, "SSLv3")) {
 			method = wolfSSLv23_client_method();
 			min_version = WOLFSSL_SSLV3;
-		} else if (!wget_strcasecmp_ascii(_config.secure_protocol, "TLSv1")) {
+		} else if (!wget_strcasecmp_ascii(config.secure_protocol, "TLSv1")) {
 			method = wolfSSLv23_client_method();
 			min_version = WOLFSSL_TLSV1;
-		} else if (!wget_strcasecmp_ascii(_config.secure_protocol, "TLSv1_1")) {
+		} else if (!wget_strcasecmp_ascii(config.secure_protocol, "TLSv1_1")) {
 			method = wolfSSLv23_client_method();
 			min_version = WOLFSSL_TLSV1_1;
-		} else if (!wget_strcasecmp_ascii(_config.secure_protocol, "TLSv1_2")) {
+		} else if (!wget_strcasecmp_ascii(config.secure_protocol, "TLSv1_2")) {
 			method = wolfSSLv23_client_method();
 			min_version = WOLFSSL_TLSV1_2;
-		} else if (!wget_strcasecmp_ascii(_config.secure_protocol, "TLSv1_3")) {
+		} else if (!wget_strcasecmp_ascii(config.secure_protocol, "TLSv1_3")) {
 			method = wolfSSLv23_client_method();
 			min_version = WOLFSSL_TLSV1_3;
-		} else if (!wget_strcasecmp_ascii(_config.secure_protocol, "PFS")) {
+		} else if (!wget_strcasecmp_ascii(config.secure_protocol, "PFS")) {
 			method = wolfSSLv23_client_method();
 			ciphers = "HIGH:!aNULL:!RC4:!MD5:!SRP:!PSK:!kRSA";
-		} else if (!wget_strcasecmp_ascii(_config.secure_protocol, "auto")) {
+		} else if (!wget_strcasecmp_ascii(config.secure_protocol, "auto")) {
 			method = wolfSSLv23_client_method();
 			min_version = WOLFSSL_TLSV1_2;
 			ciphers = "HIGH:!aNULL:!RC4:!MD5:!SRP:!PSK";
-		} else if (*_config.secure_protocol) {
+		} else if (*config.secure_protocol) {
 			method = wolfSSLv23_client_method();
-			ciphers = _config.secure_protocol;
+			ciphers = config.secure_protocol;
 		} else {
 			error_printf(_("Missing TLS method\n"));
 			goto out;
@@ -610,25 +646,33 @@ void wget_ssl_init(void)
 			if (!wolfSSL_CTX_set_cipher_list(ssl_ctx, ciphers))
 				error_printf(_("WolfSSL: Failed to set ciphers '%s'\n"), ciphers);
 
-		if (_config.check_certificate) {
-			if (!wget_strcmp(_config.ca_directory, "system"))
-				_config.ca_directory = "/etc/ssl/certs";
-
-			/* Load client certificates into WOLFSSL_CTX */
-			if (wolfSSL_CTX_load_verify_locations(ssl_ctx, _config.ca_file, _config.ca_directory) != SSL_SUCCESS) {
-				error_printf(_("Failed to load %s, please check the file.\n"), _config.ca_directory);
+		if (config.check_certificate) {
+			if (!wget_strcmp(config.ca_directory, "system"))
+				config.ca_directory = wget_ssl_default_cert_dir();
+			if (config.ca_file && !wget_strcmp(config.ca_file, "system"))
+				config.ca_file = wget_ssl_default_ca_bundle_path();
+			const char* dir = config.ca_directory;
+			const char* file = config.ca_file;
+			if (dir && access(dir, F_OK))
+				dir = NULL;
+			else if (file && access(file, F_OK)) //yes else if, good to throw an error if neither are there, just don't want to do it if at least one exists
+				file = NULL;
+			// Load client certificates into WOLFSSL_CTX
+			if (wolfSSL_CTX_load_verify_locations(ssl_ctx, file, dir) != SSL_SUCCESS) {
+				error_printf(_("Failed to load CA pem: %s or cert dir: %s, ssl verification will likely fail.\n"), config.ca_file, config.ca_directory);
 				goto out;
 			}
+			wolfSSL_CTX_set_verify(ssl_ctx, SSL_VERIFY_PEER, NULL);
 		} else {
 			wolfSSL_CTX_set_verify(ssl_ctx, SSL_VERIFY_NONE, NULL);
 		}
 
-/*		if (_config.crl_file) {
+/*		if (config.crl_file) {
 			WOLFSSL_X509_STORE *store = wolfSSL_CTX_get_cert_store(ssl_ctx);
 			WOLFSSL_X509_LOOKUP *lookup;
 
 			if (!(lookup = wolfSSL_X509_STORE_add_lookup(store, wolfSSL_X509_LOOKUP_file()))
-				|| (!X509_load_crl_file(lookup, _config.crl_file, X509_FILETYPE_PEM)))
+				|| (!X509_load_crl_file(lookup, config.crl_file, X509_FILETYPE_PEM)))
 				return;
 
 			wolfSSL_X509_STORE_set_flags(store, WOLFSSL_CRL_CHECK | WOLFSSL_CRL_CHECKALL);
@@ -637,12 +681,12 @@ void wget_ssl_init(void)
 		debug_printf("Certificates loaded\n");
 
 out:
-		_init++;
+		init++;
 
 		debug_printf("WolfSSL init done\n");
 	}
 
-	wget_thread_mutex_unlock(_mutex);
+	wget_thread_mutex_unlock(mutex);
 }
 
 /**
@@ -659,19 +703,19 @@ out:
  */
 void wget_ssl_deinit(void)
 {
-	wget_thread_mutex_lock(_mutex);
+	wget_thread_mutex_lock(mutex);
 
-	if (_init == 1) {
+	if (init == 1) {
 		wolfSSL_CTX_free(ssl_ctx); ssl_ctx = NULL;
 		wolfSSL_Cleanup();
 	}
 
-	if (_init > 0) _init--;
+	if (init > 0) init--;
 
-	wget_thread_mutex_unlock(_mutex);
+	wget_thread_mutex_unlock(mutex);
 }
 
-static int _do_handshake(WOLFSSL *session, int sockfd, int timeout)
+static int do_handshake(WOLFSSL *session, int sockfd, int timeout)
 {
 	int ret;
 
@@ -756,20 +800,16 @@ static void ShowX509(WOLFSSL_X509 *x509, const char *hdr)
 
 	ret = wolfSSL_X509_get_serial_number(x509, serial, &sz);
 	if (ret == WOLFSSL_SUCCESS) {
-		int i;
-		int strLen;
-		char serialMsg[80];
-
-		/* testsuite has multiple threads writing to stdout, get output
-			message ready to write once */
-		strLen = sprintf(serialMsg, " serial number");
-		for (i = 0; i < sz; i++)
-			sprintf(serialMsg + strLen + (i * 3), ":%02x ", serial[i]);
-		debug_printf("%s\n", serialMsg);
+		char serialMsg[sizeof(serial) * 4 + 1];
+		// testsuite has multiple threads writing to stdout, get output
+		// message ready to write once
+		for (int i = 0; i < sz; i++)
+			sprintf(serialMsg + (i * 4), ":%02x ", serial[i]);
+		debug_printf(" serial number%s\n", serialMsg);
 	}
 
-	XFREE(subject, 0, DYNAMIC_TYPE_OPENSSL);
-	XFREE(issuer, 0, DYNAMIC_TYPE_OPENSSL);
+	XFREE(subject, 0, DYNAMIC_TYPE_OPENSSL)
+	XFREE(issuer, 0, DYNAMIC_TYPE_OPENSSL)
 
 	{
 		WOLFSSL_BIO* bio;
@@ -788,7 +828,8 @@ static void ShowX509(WOLFSSL_X509 *x509, const char *hdr)
 		bio = wolfSSL_BIO_new(wolfSSL_BIO_s_file());
 		if (bio) {
 			wolfSSL_BIO_set_fp(bio, stdout, BIO_NOCLOSE);
-			wolfSSL_X509_print(bio, x509);
+			if (wget_logger_is_active(wget_get_logger(WGET_LOGGER_DEBUG)))
+				wolfSSL_X509_print(bio, x509);
 			wolfSSL_BIO_free(bio);
 		}
 	}
@@ -848,7 +889,7 @@ int wget_ssl_open(wget_tcp *tcp)
 	if (!tcp)
 		return WGET_E_INVALID;
 
-	if (!_init)
+	if (!init)
 		wget_ssl_init();
 
 	hostname = tcp->ssl_hostname;
@@ -862,29 +903,33 @@ int wget_ssl_open(wget_tcp *tcp)
 
 	// RFC 6066 SNI Server Name Indication
 	if (hostname)
-		wolfSSL_UseSNI(session, WOLFSSL_SNI_HOST_NAME, hostname, strlen(hostname));
+		wolfSSL_UseSNI(session, WOLFSSL_SNI_HOST_NAME, hostname, (unsigned short) strlen(hostname));
 
 //	if (tcp->tls_false_start)
 //		info_printf(_("WolfSSL doesn't support TLS False Start\n"));
 
-	if (_config.alpn) {
-		size_t len = strlen(_config.alpn);
-		char alpn[len + 1];
+	if (config.alpn) {
+		size_t len = strlen(config.alpn);
+		char alpnbuf[256], *alpn;
 
 		// wolfSSL_UseALPN() destroys the ALPN string (bad design pattern !)
-		memcpy(alpn, _config.alpn, len + 1);
-		if (wolfSSL_UseALPN(session, alpn, len, WOLFSSL_ALPN_CONTINUE_ON_MISMATCH) == WOLFSSL_SUCCESS) {
-			debug_printf("ALPN offering %s\n", _config.alpn);
+		alpn = wget_strmemcpy_a(alpnbuf, sizeof(alpnbuf), config.alpn, strlen(config.alpn));
+
+		if (wolfSSL_UseALPN(session, alpn, (int) len, WOLFSSL_ALPN_CONTINUE_ON_MISMATCH) == WOLFSSL_SUCCESS) {
+			debug_printf("ALPN offering %s\n", config.alpn);
 		} else
-			debug_printf("WolfSSL: Failed to set ALPN: %s\n", _config.alpn);
+			debug_printf("WolfSSL: Failed to set ALPN: %s\n", config.alpn);
+
+		if (alpn != alpnbuf)
+			xfree(alpn);
 	}
 
-	struct _session_context *ctx = wget_calloc(1, sizeof(struct _session_context));
-	ctx->hostname = wget_strdup(hostname);
+	// struct session_context *ctx = wget_calloc(1, sizeof(struct session_context));
+	// ctx->hostname = wget_strdup(hostname);
 
 	tcp->ssl_session = session;
 //	gnutls_session_set_ptr(session, ctx);
-	wolfSSL_set_fd(session, sockfd);
+	wolfSSL_set_fd(session, FD_TO_SOCKET(sockfd));
 
 	/* make wolfSSL object nonblocking */
 	wolfSSL_set_using_nonblock(session, 1);
@@ -892,7 +937,7 @@ int wget_ssl_open(wget_tcp *tcp)
 	if (tls_stats_callback)
 		before_millisecs = wget_get_timemillis();
 
-	ret = _do_handshake(session, sockfd, connect_timeout);
+	ret = do_handshake(session, sockfd, connect_timeout);
 
 	if (tls_stats_callback) {
 		long long after_millisecs = wget_get_timemillis();
@@ -922,7 +967,7 @@ int wget_ssl_open(wget_tcp *tcp)
 	else if ((bits = wolfSSL_GetDhKey_Sz(session)) > 0)
 		debug_printf("SSL DH size %d bits\n", bits);
 
-	if (_config.alpn) {
+	if (config.alpn) {
 		char *protocol;
 		uint16_t protocol_length;
 
@@ -930,7 +975,8 @@ int wget_ssl_open(wget_tcp *tcp)
 			debug_printf("WolfSSL: Failed to connect ALPN\n");
 		else {
 			debug_printf("WolfSSL: Server accepted ALPN protocol '%.*s'\n", (int) protocol_length, protocol);
-			stats.alpn_protocol = wget_strmemdup(protocol, protocol_length);
+			if (tls_stats_callback)
+				stats.alpn_protocol = wget_strmemdup(protocol, protocol_length);
 
 			if (protocol_length == 2 && !memcmp(protocol, "h2", 2)) {
 				tcp->protocol = WGET_PROTOCOL_HTTP_2_0;
@@ -943,12 +989,11 @@ int wget_ssl_open(wget_tcp *tcp)
 		int resumed = wolfSSL_session_reused(session);
 
 		WOLFSSL_X509_CHAIN *chain = (WOLFSSL_X509_CHAIN *) wolfSSL_get_peer_cert_chain(session);
-		stats.cert_chain_size = wolfSSL_get_chain_count(chain);
-//		debug_printf("Cert chain size %d\n", stats.cert_chain_size);
 		ShowX509Chain(chain, wolfSSL_get_chain_count(chain), "Certificate chain");
 
 		if (tls_stats_callback) {
 			stats.resumed = resumed;
+			stats.cert_chain_size = wolfSSL_get_chain_count(chain);
 
 			const char *tlsver = wolfSSL_get_version(session);
 			if (!strcmp(tlsver, "TLSv1.2"))
@@ -957,23 +1002,18 @@ int wget_ssl_open(wget_tcp *tcp)
 				stats.version = 5;
 			else
 				stats.version = 1; // SSLv3
-			// stats.version = gnutls_protocol_get_version(session);
-
-			WOLFSSL_X509_CHAIN *chain = (WOLFSSL_X509_CHAIN *) wolfSSL_get_peer_cert_chain(session);
-			stats.cert_chain_size = wolfSSL_get_chain_count(chain);
-			// gnutls_certificate_get_peers(session, (unsigned int *)&(stats.cert_chain_size));
 		}
 
 		debug_printf("Handshake completed%s\n", resumed ? " (resumed session)" : "");
 
-		if (!resumed && _config.tls_session_cache) {
+		if (!resumed && config.tls_session_cache) {
 /*			WOLFSSL_SESSION *session_data = wolfSSL_get_session(session);
 
 			if (session_data) {
 				int session_data_size = wolfSSL_get_session_cache_memsize();
 				char session_data_data[session_data_size];
 				if (wolfSSL_memsave_session_cache(session_data_data, session_data_size) == SSL_SUCCESS) {
-					wget_tls_session_db_add(_config.tls_session_cache,
+					wget_tls_session_db_add(config.tls_session_cache,
 						wget_tls_session_new(ctx->hostname, 18 * 3600, session_data.data, session_data.size)); // 18h valid
 				}
 			}
@@ -981,7 +1021,7 @@ int wget_ssl_open(wget_tcp *tcp)
 /*			gnutls_datum_t session_data;
 
 			if ((rc = gnutls_session_get_data2(session, &session_data)) == GNUTLS_E_SUCCESS) {
-				wget_tls_session_db_add(_config.tls_session_cache,
+				wget_tls_session_db_add(config.tls_session_cache,
 					wget_tls_session_new(ctx->hostname, 18 * 3600, session_data.data, session_data.size)); // 18h valid
 				gnutls_free(session_data.data);
 			} else
@@ -993,8 +1033,8 @@ int wget_ssl_open(wget_tcp *tcp)
 		rc = wolfSSL_get_error(session, rc);
 		error_printf(_("failed to connect TLS (%d): %s\n"), rc, wolfSSL_ERR_reason_error_string(rc));
 
-		rc = wolfSSL_get_verify_result(session);
-		if (rc >= 13 && rc <= 29)
+		long res = wolfSSL_get_verify_result(session);
+		if (res >= 13 && res <= 29)
 			return WGET_E_CERTIFICATE;
 		else
 			return WGET_E_CONNECT;
@@ -1006,13 +1046,13 @@ int wget_ssl_open(wget_tcp *tcp)
 		xfree(stats.alpn_protocol);
 	}
 
-	tcp->hpkp = ctx->stats_hpkp;
+	// tcp->hpkp = ctx->stats_hpkp;
 
 	if (ret != WGET_E_SUCCESS) {
 		if (ret == WGET_E_TIMEOUT)
 			debug_printf("Handshake timed out\n");
-		xfree(ctx->hostname);
-		xfree(ctx);
+		// xfree(ctx->hostname);
+		// xfree(ctx);
 		wolfSSL_free(session);
 		tcp->ssl_session = NULL;
 	}
@@ -1070,10 +1110,10 @@ void wget_ssl_close(void **session)
  */
 ssize_t wget_ssl_read_timeout(void *session, char *buf, size_t count, int timeout)
 {
-	int sockfd = wolfSSL_get_fd(session);
+	int sockfd = SOCKET_TO_FD( wolfSSL_get_fd(session));
 	int rc;
 
-	while ((rc = wolfSSL_read(session, buf, count)) < 0) {
+	while ((rc = wolfSSL_read(session, buf, (int) count)) < 0) {
 		rc =  wolfSSL_get_error(session, rc);
 		debug_printf("wolfSSL_read: (%d) (errno=%d) %s\n", rc, errno, wolfSSL_ERR_reason_error_string(rc));
 		if (rc == SSL_ERROR_WANT_READ) {
@@ -1094,14 +1134,14 @@ ssize_t wget_ssl_read_timeout(void *session, char *buf, size_t count, int timeou
 		nbytes = gnutls_record_recv(session, buf, count);
 
 		// If False Start + Session Resumption are enabled, we get the session data after the first read()
-		struct _session_context *ctx = gnutls_session_get_ptr(session);
+		struct session_context *ctx = gnutls_session_get_ptr(session);
 		if (ctx && ctx->delayed_session_data) {
 			gnutls_datum_t session_data;
 
 			if ((rc = gnutls_session_get_data2(session, &session_data)) == GNUTLS_E_SUCCESS) {
 				debug_printf("Got delayed session data\n");
 				ctx->delayed_session_data = 0;
-				wget_tls_session_db_add(_config.tls_session_cache,
+				wget_tls_session_db_add(config.tls_session_cache,
 					wget_tls_session_new(ctx->hostname, 18 * 3600, session_data.data, session_data.size)); // 18h valid
 				gnutls_free(session_data.data);
 			} else
@@ -1110,7 +1150,7 @@ ssize_t wget_ssl_read_timeout(void *session, char *buf, size_t count, int timeou
 
 		if (nbytes == GNUTLS_E_REHANDSHAKE) {
 			debug_printf("*** REHANDSHAKE while reading\n");
-			if ((nbytes = _do_handshake(session, sockfd, timeout)) == 0)
+			if ((nbytes = do_handshake(session, sockfd, timeout)) == 0)
 				nbytes = GNUTLS_E_AGAIN; // restart reading
 		}
 		if (nbytes >= 0 || nbytes != GNUTLS_E_AGAIN)
@@ -1143,10 +1183,10 @@ ssize_t wget_ssl_read_timeout(void *session, char *buf, size_t count, int timeou
  */
 ssize_t wget_ssl_write_timeout(void *session, const char *buf, size_t count, int timeout)
 {
-	int sockfd = wolfSSL_get_fd(session);
+	int sockfd = SOCKET_TO_FD(wolfSSL_get_fd(session));
 	int rc;
 
-	while ((rc = wolfSSL_write(session, buf, count)) < 0) {
+	while ((rc = wolfSSL_write(session, buf, (int) count)) < 0) {
 		rc =  wolfSSL_get_error(session, rc);
 		debug_printf("wolfSSL_write: (%d) (errno=%d) %s\n", rc, errno, wolfSSL_ERR_reason_error_string(rc));
 		if (rc == SSL_ERROR_WANT_WRITE) {
@@ -1170,7 +1210,7 @@ ssize_t wget_ssl_write_timeout(void *session, const char *buf, size_t count, int
 
 		if (nbytes == GNUTLS_E_REHANDSHAKE) {
 			debug_printf("*** REHANDSHAKE while writing\n");
-			if ((nbytes = _do_handshake(session, sockfd, timeout)) == 0)
+			if ((nbytes = do_handshake(session, sockfd, timeout)) == 0)
 				continue; // restart writing
 		}
 		if (nbytes == GNUTLS_E_AGAIN)
