@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2022 Free Software Foundation, Inc.
+ * Copyright (c) 2016-2023 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -365,12 +365,24 @@ static int tls_session_save(void *_fp, const void *_tls_session, WGET_GCC_UNUSED
 {
 	FILE *fp = _fp;
 	const wget_tls_session *tls_session = _tls_session;
+	char tmp[1024], *session_b64 = tmp;
+	size_t b64_len = wget_base64_get_encoded_length(tls_session->data_size);
 
-	char session_b64[wget_base64_get_encoded_length(tls_session->data_size)];
+	if (b64_len > sizeof(tmp)) {
+		session_b64 = wget_malloc(b64_len);
+		if (!session_b64) {
+			error_printf(_("Failed to allocate %zu bytes\n"), b64_len);
+			return 0;
+		}
+	}
 
 	wget_base64_encode(session_b64, (const char *) tls_session->data, tls_session->data_size);
 
 	wget_fprintf(fp, "%s %lld %lld %s\n", tls_session->host, (long long)tls_session->created, (long long)tls_session->maxage, session_b64);
+
+	if (session_b64 != tmp)
+		xfree(session_b64);
+
 	return 0;
 }
 
