@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2018-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2018-2023 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -38,13 +38,7 @@ void mkdir_path(const char *_fname, bool is_file)
 	char *p2, *fname;
 	char buf[1024];
 
-	size_t len = strlen(_fname);
-
-	if (len < sizeof(buf)) {
-		memcpy(buf, _fname, len + 1);
-		fname = buf;
-	} else
-		fname = wget_strdup(_fname);
+	fname = wget_strmemcpy_a(buf, sizeof(buf), _fname, strlen(_fname));
 
 	for (p1 = fname + 1; *p1 && (p2 = strchr(p1, '/')); p1 = p2 + 1) {
 		int rc;
@@ -57,7 +51,9 @@ void mkdir_path(const char *_fname, bool is_file)
 
 		rc = mkdir(fname, 0755);
 
-		debug_printf("mkdir(%s)=%d errno=%d\n",fname,rc,errno);
+		if (rc < 0 && errno != EEXIST)
+			debug_printf("mkdir(%s)=%d errno=%d\n", fname, rc, errno);
+
 		if (rc) {
 			struct stat st;
 
@@ -66,11 +62,17 @@ void mkdir_path(const char *_fname, bool is_file)
 				int renamed = 0;
 
 				for (int fnum = 1; fnum <= 999 && !renamed; fnum++) {
-					char dst[strlen(fname) + 1 + 32];
+					char tmp[1024], *dst = tmp;
+
+					if (wget_snprintf(tmp, sizeof(tmp), "%s.%d", fname, fnum) >= sizeof(tmp)) {
+						dst = wget_aprintf("%s.%d", fname, fnum);
+					}
 
-					wget_snprintf(dst, sizeof(dst), "%s.%d", fname, fnum);
 					if (access(dst, F_OK) != 0 && rename(fname, dst) == 0)
 						renamed = 1;
+
+					if (dst != tmp)
+						xfree(dst);
 				}
 
 				if (renamed) {
@@ -100,7 +102,8 @@ void mkdir_path(const char *_fname, bool is_file)
 	// directory for the last part
 	if (!is_file) {
 		int rc = mkdir(_fname, 0755);
-		debug_printf("mkdir(%s)=%d errno=%d\n",_fname,rc,errno);
+		if (rc < 0 && errno != EEXIST)
+			debug_printf("mkdir(%s)=%d errno=%d\n", _fname, rc, errno);
 	}
 }
 
