@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -30,12 +30,23 @@
 
 #include <wget.h>
 
+typedef struct {
+	const wget_iri
+		*iri;
+	char *
+		local_filename;
+} blacklist_entry;
+
 void blacklist_init(void);
 void blacklist_exit(void);
-int in_blacklist(wget_iri_t *iri) G_GNUC_WGET_NONNULL_ALL;
-int blacklist_size(void) G_GNUC_WGET_PURE;
-wget_iri_t *blacklist_add(wget_iri_t *iri);
+int blacklist_size(void) WGET_GCC_PURE;
+blacklist_entry *blacklist_add(const wget_iri *iri);
+blacklist_entry *blacklist_get(const wget_iri *iri);
 void blacklist_print(void);
 void blacklist_free(void);
+void blacklist_set_filename(blacklist_entry *blacklistp, const char *fname);
+
+WGET_GCC_NONNULL_ALL
+char *get_local_filename(const wget_iri *iri);
 
 #endif /* SRC_WGET_BLACKLIST_H */
