@@ -1,21 +1,21 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing Wget
@@ -53,6 +53,13 @@ int main(void)
 			.headers = {
 				"Content-Type: text/html",
 			}
+		},
+		{	.name = "/error503.html",
+			.code = "503 Service Unavailable",
+			.body = "503 error page",
+			.headers = {
+				"Content-Type: text/html",
+			}
 		}
 	};
 
@@ -79,6 +86,7 @@ int main(void)
 			{	NULL } },
 		0);
 
+#ifndef WITH_WOLFSSL
 	// test-i-https with loading CA Certificate and CRL
 	wget_test(
 		// WGET_TEST_KEEP_TMPFILES, 1,
@@ -92,6 +100,7 @@ int main(void)
 			{ urls[0].name + 1, urls[0].body },
 			{	NULL } },
 		0);
+#endif
 
 	// test-i-https ignoring unknown certificate
 	wget_test(
@@ -109,6 +118,7 @@ int main(void)
 			{	NULL } },
 		0);
 
+#ifndef WITH_WOLFSSL
 	// test-i-https ignoring unknown certificate (with CRL)
 	wget_test(
 		// WGET_TEST_KEEP_TMPFILES, 1,
@@ -124,11 +134,12 @@ int main(void)
 			{ urls[2].name + 1, urls[2].body },
 			{	NULL } },
 		0);
+#endif
 
 	// test-i-https failing due to unknown certificate
 	wget_test(
 		// WGET_TEST_KEEP_TMPFILES, 1,
-		WGET_TEST_OPTIONS, "--tries=1 -i urls.txt",
+		WGET_TEST_OPTIONS, "--tries=1 --no-ca-certificate -i urls.txt",
 		WGET_TEST_REQUEST_URL, NULL,
 		WGET_TEST_EXPECTED_ERROR_CODE, 5,
 		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
@@ -139,6 +150,15 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test response code 503
+	wget_test(
+		WGET_TEST_OPTIONS, "",
+		WGET_TEST_REQUEST_URL, "error503.html",
+		WGET_TEST_EXPECTED_ERROR_CODE, 8,
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	NULL } },
+		0);
+
 /*
 	// test-i-http (expands to -i https://localhost:{{sslport}}/urls.txt)
 	wget_test(
@@ -152,5 +172,5 @@ int main(void)
 			{	NULL } },
 		0);
 */
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
