@@ -1,20 +1,20 @@
 /*
- * Copyright(c) 2016-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2016-2023 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Test Metalink functionality
@@ -28,6 +28,7 @@
 #include <config.h>
 
 #include <stdlib.h> // exit()
+#include <string.h>
 #include "libtest.h"
 
 int main(void)
@@ -119,9 +120,9 @@ int main(void)
 		},
 	};
 
-	int hashlen = wget_hash_get_len(WGET_DIGTYPE_MD5);
-	char md5hex[hashlen * 2 + 1], md5hex_p1[hashlen * 2 + 1], md5hex_p2[hashlen * 2 + 1];
-	unsigned char digest[hashlen];
+	// int hashlen = wget_hash_get_len(WGET_DIGTYPE_MD5); // 20
+	char md5hex[20 * 2 + 1], md5hex_p1[20 * 2 + 1], md5hex_p2[20 * 2 + 1];
+	unsigned char digest[20];
 	char *body0, *body2, *body4, *body7; // to be freed later
 
 	wget_hash_printf_hex(WGET_DIGTYPE_MD5, md5hex, sizeof(md5hex), "%s", urls[1].body);
@@ -200,6 +201,7 @@ int main(void)
 	wget_test_start_server(
 		WGET_TEST_RESPONSE_URLS, &urls, countof(urls),
 		WGET_TEST_FEATURE_MHD,
+		WGET_TEST_SKIP_H2,
 		0);
 
 	// --no-metalink
@@ -280,7 +282,7 @@ int main(void)
 			{ NULL } },
 		0);
 
-	free(digest_str);
+	wget_free(digest_str);
 
 	/**** RFC 6249 Metalink/HTTP: Mirrors and Hashes  - without metalink description ****/
 	wget_hash_fast(WGET_DIGTYPE_MD5, urls[10].body, strlen(urls[10].body), digest);
@@ -296,14 +298,14 @@ int main(void)
 			{ NULL } },
 		0);
 
-	free(digest_str);
+	wget_free(digest_str);
 
-	free(body0);
-	free(body2);
-	free(body4);
-	free(body7);
-	free((void *)urls[6].headers[4]);
-	free((void *)urls[9].headers[3]);
+	wget_free(body0);
+	wget_free(body2);
+	wget_free(body4);
+	wget_free(body7);
+	wget_free((void *)urls[6].headers[4]);
+	wget_free((void *)urls[9].headers[3]);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
