@@ -1,20 +1,20 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2023 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing HTML CSS parsing
@@ -31,15 +31,16 @@ int main(void)
 	wget_test_url_t urls[]={
 		{	.name = "/page1.html",
 			.code = "200 Dontcare",
-			.body =	"<html>\n"\
-						"<head>\n"\
-						"  <style> #test1{background:url(/page2.html) no-repeat center} </style>\n"\
-						"  <style> #test2{background:url(\"/page3.html\") no-repeat center} </style>\n"\
-						"</head>\n"\
-						"<body>\n"\
-						"  <div style=\"background:url(/page4.html) no-repeat center; \"></div>\n"\
-						"</body>\n"\
-						"</html>\n",
+			.body =
+				"<html>\n"\
+				"<head>\n"\
+				"  <style> #test1{background:url(/page2.html) no-repeat center} </style>\n"\
+				"  <style> #test2{background:url(\"/page3.html\") no-repeat center} </style>\n"\
+				"</head>\n"\
+				"<body>\n"\
+				"  <div style=\"background:url(/page4.html) no-repeat center; \"></div>\n"\
+				"</body>\n"\
+				"</html>\n",
 			.headers = {
 				"Content-Type: text/html",
 			}
@@ -87,10 +88,8 @@ int main(void)
 		0);
 
 	char options[128];
-	snprintf(options, sizeof(options),
-		"--force-html --base http://localhost:%d -i page1.html",
-		wget_test_get_http_server_port());
-
+	wget_snprintf(options, sizeof(options),
+		"--force-html --base http://localhost:{{port}} -i page1.html");
 
 	// test--parse-html-css from file
 	wget_test(
@@ -108,5 +107,5 @@ int main(void)
 			{	NULL } },
 		0);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
