@@ -1,20 +1,20 @@
 /*
- * Copyright(c) 2017-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2023 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Dummy plugins for plugin support testing
@@ -36,7 +36,7 @@ int wget_plugin_initializer(wget_plugin *plugin)
 	const char *name = wget_plugin_get_name(plugin);
 	if (strcmp(name, "pluginname") != 0) {
 		wget_error_printf("Plugin took a wrong name '%s'\n", name);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	FILE *stream = fopen("plugin-loaded.txt", "wb");
@@ -588,7 +588,7 @@ static void test_hsts_db_free(wget_hsts_db **hsts_db)
 	*hsts_db = NULL;
 }
 
-static void test_hsts_db_add(wget_hsts_db *hsts_db, const char *host, uint16_t port, time_t maxage, int include_subdomains)
+static void test_hsts_db_add(wget_hsts_db *hsts_db, const char *host, uint16_t port, int64_t maxage, bool include_subdomains)
 {
 	(void) hsts_db;
 	wget_debug_printf("%s: host %s port %hu maxage %lld include_subdomains %d\n", __func__,
@@ -669,14 +669,14 @@ static bool test_ocsp_db_hostname_is_valid(const wget_ocsp_db *ocsp_db, const ch
 	return true;
 }
 
-static void test_ocsp_db_add_fingerprint(wget_ocsp_db *ocsp_db, const char *fingerprint, time_t maxage, bool valid)
+static void test_ocsp_db_add_fingerprint(wget_ocsp_db *ocsp_db, const char *fingerprint, int64_t maxage, bool valid)
 {
 	(void) ocsp_db;
 
 	wget_debug_printf("%s: fingerprint %s maxage %lld valid %d\n", __func__, fingerprint, (long long) maxage, valid);
 }
 
-static void test_ocsp_db_add_host(wget_ocsp_db *ocsp_db, const char *host, time_t maxage)
+static void test_ocsp_db_add_host(wget_ocsp_db *ocsp_db, const char *host, int64_t maxage)
 {
 	(void) ocsp_db;
 
