@@ -1,20 +1,20 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2023 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing Wget plugin support
@@ -27,27 +27,10 @@
 #include <string.h> // strlen()
 #include <unistd.h> // access()
 #include "libtest.h"
+#include "plugin_tests.h"
 
 // #define LARGEFILE (11 << 20)
 
-#define OBJECT_DIR "../.libs"
-
-#if defined _WIN32
-#define LOCAL_NAME(x) OBJECT_DIR "/lib" x ".dll"
-#elif defined __CYGWIN__
-#define LOCAL_NAME(x) OBJECT_DIR "/cyg" x ".dll"
-#else
-#define LOCAL_NAME(x) OBJECT_DIR "/lib" x ".so"
-#endif
-
-#ifdef _WIN32
-#define setenv_rpl(name, value, ignored) _putenv(name "=" value)
-#define unsetenv_rpl(name) _putenv(name "=")
-#else
-#define setenv_rpl setenv
-#define unsetenv_rpl unsetenv
-#endif
-
 static const char *mainpage = "\
 <html>\n\
 <head>\n\
@@ -194,7 +177,7 @@ int main(void)
 	if (access(".libs/libpluginname.so", R_OK) != 0
 		&& access(".libs/libpluginname.dll", R_OK) != 0
 		&& access(".libs/cygpluginname.dll", R_OK) != 0)
-		exit(77); // likely a static build
+		exit(WGET_TEST_EXIT_SKIP); // likely a static build
 
 	wget_test_start_server(
 			WGET_TEST_RESPONSE_URLS, &urls, countof(urls),
@@ -369,5 +352,5 @@ int main(void)
 			{ NULL } },
 		0);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
