@@ -1,20 +1,20 @@
 /*
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing basic robots.txt functionality
@@ -62,6 +62,7 @@ int main(void)
 				" <A href=\"../secondpage.html\">second page</a>." \
 				" <a href=\"subpage1.html?query&param#frag\">page in subdir2</a>." \
 				" <a href=\"./subpage2.html\">page in subdir2</a>." \
+				" <a href=\"./audio.mp3\">mp3 in subdir2</a>." \
 				"</p></body></html>",
 			.headers = {
 				"Content-Type: text/html",
@@ -83,6 +84,20 @@ int main(void)
 			.code = "200 Dontcare",
 			.body = "sub2_2"
 		},
+		{	.name = "/subdir2/audio.mp3",
+			.code = "200 Dontcare",
+			.body = "sub2_audio.mp3"
+		},
+		{	.name = "/robots_open.txt",
+			.code = "200 Dontcare",
+			.body =
+				"User-agent: *\n"\
+				"Allow: /\n"\
+			,
+			.headers = {
+				"Content-Type: text/plain",
+			}
+		},
 	};
 
 	// functions won't come back if an error occurs
@@ -91,6 +106,43 @@ int main(void)
 		WGET_TEST_FEATURE_MHD,
 		0);
 
+	// Check if robots is actually honoured when it already exists on the
+	// file system
+	wget_test(
+		WGET_TEST_OPTIONS, "-r -nH",
+		WGET_TEST_REQUEST_URL, "index.html",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ "robots.txt", urls[7].body },
+			{	NULL } },
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{ urls[1].name + 1, urls[1].body },
+			{ urls[2].name + 1, urls[2].body },
+			{ urls[3].name + 1, urls[3].body },
+			{ urls[4].name + 1, urls[4].body },
+			{	NULL } },
+		0);
+
+
+	// Check if robots is actually honoured when it already exists on the
+	// file system
+	wget_test(
+		WGET_TEST_OPTIONS, "-r -nH -c",
+		WGET_TEST_REQUEST_URL, "index.html",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ "robots.txt", urls[0].body },
+			{	NULL } },
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{ urls[1].name + 1, urls[1].body },
+			{ urls[2].name + 1, urls[2].body },
+			{ urls[3].name + 1, urls[3].body },
+			{ urls[4].name + 1, urls[4].body },
+			{	NULL } },
+		0);
+
 	// robots.txt forbids /subdir2/ for '*'
 	wget_test(
 		WGET_TEST_OPTIONS, "-r -nH",
@@ -119,5 +171,23 @@ int main(void)
 			{ urls[6].name + 1, urls[6].body },
 			{	NULL } },
 		0);
-	exit(0);
+
+	// Disable robots.txt
+	wget_test(
+		WGET_TEST_OPTIONS, "-r --no-robots -nH",
+		WGET_TEST_REQUEST_URL, "index.html",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{ urls[1].name + 1, urls[1].body },
+			{ urls[2].name + 1, urls[2].body },
+			{ urls[3].name + 1, urls[3].body },
+			{ urls[4].name + 1, urls[4].body },
+			{ urls[5].name + 1, urls[5].body },
+			{ urls[6].name + 1, urls[6].body },
+			{ urls[7].name + 1, urls[7].body },
+			{	NULL } },
+		0);
+
+	exit(EXIT_SUCCESS);
 }
