@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2022 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2023 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -311,7 +311,7 @@ int main(WGET_GCC_UNUSED int argc, char **argv)
 		// fallthrough
 	}
 	else if (!strcmp(valgrind, "1")) {
-		char cmd[strlen(argv[0]) + 256];
+		char cmd[4096];
 
 		wget_snprintf(cmd, sizeof(cmd), "VALGRIND_TESTS=\"\" valgrind "
 				"--error-exitcode=301 --leak-check=yes "
@@ -319,7 +319,7 @@ int main(WGET_GCC_UNUSED int argc, char **argv)
 				argv[0]);
 		return system(cmd) != 0;
 	} else {
-		char cmd[strlen(valgrind) + strlen(argv[0]) + 32];
+		char cmd[4096];
 
 		wget_snprintf(cmd, sizeof(cmd), "VALGRIND_TESTS="" %s %s",
 				valgrind, argv[0]);
