@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2023 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -39,6 +39,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <c-ctype.h>
+#include <errno.h>
 
 #include <wget.h>
 #include "../libwget/private.h"
@@ -56,7 +57,7 @@ static void check(int result, int line, const char *msg)
 		ok++;
 	} else {
 		failed++;
-		wget_info_printf("L%d: %s\n", line, msg);
+		wget_error_printf_exit("L%d: %s\n", line, msg);
 	}
 }
 
@@ -83,26 +84,28 @@ static void test_mem(void)
 
 	CHECK(!wget_strmemdup(NULL, 0));
 	CHECK(p = wget_strmemdup("xxx", 1));
-	CHECK(!strcmp(p, "x")); xfree(p);
+	CHECK(!memcmp(p, "x", 1)); xfree(p);
 	CHECK(p = wget_strmemdup("xxx", 0));
 	xfree(p);
 
-	wget_strmemcpy(NULL, 0, NULL, 0);
-	wget_strmemcpy(NULL, 5, NULL, 3);
+	CHECK(wget_strmemcpy(NULL, 0, NULL, 0) == 0);
+	CHECK(wget_strmemcpy(NULL, 5, NULL, 3) == 0);
 
 	char buf[32] = "x";
-	wget_strmemcpy(buf, 0, "xxx", 0);
+	CHECK(wget_strmemcpy(buf, 0, "xxx", 0) == 0);
 	CHECK(!strcmp(buf, "x"));
-	wget_strmemcpy(buf, 0, "xxx", 1);
+	CHECK(wget_strmemcpy(buf, 0, "xxx", 1) == 0);
 	CHECK(!strcmp(buf, "x"));
-	wget_strmemcpy(buf, sizeof(buf), "xxx", 0);
+	CHECK(wget_strmemcpy(buf, sizeof(buf), "xxx", 0) == 0);
 	CHECK(!strcmp(buf, ""));
-	wget_strmemcpy(buf, 1, "xxx", 3);
+	CHECK(wget_strmemcpy(buf, 1, "xxx", 3) == 0);
 	CHECK(!strcmp(buf, ""));
-	wget_strmemcpy(buf, 2, "xxx", 3);
+	CHECK(wget_strmemcpy(buf, 2, "xxx", 3) == 1);
 	CHECK(!strcmp(buf, "x"));
-	wget_strmemcpy(buf, 2, NULL, 3);
+	CHECK(wget_strmemcpy(buf, 2, NULL, 3) == 0);
 	CHECK(!strcmp(buf, ""));
+	CHECK(wget_strmemcpy(buf, sizeof(buf), "xxx", 3) == 3);
+	CHECK(!strcmp(buf, "xxx"));
 
 #if __GNUC__ >= 7
 #pragma GCC diagnostic pop
@@ -270,11 +273,12 @@ static void test_buffer(void)
 	wget_buffer_init(&buf, sbuf, sizeof(sbuf));
 	for (int mid_ws = 0; mid_ws <= 2; mid_ws++) {
 		char expected[16];
-		snprintf(expected, sizeof(expected), "x%.*sy", mid_ws, "  ");
+		snprintf(expected, sizeof(expected), "x%*.*sy", mid_ws, mid_ws, "");
 
 		for (int lead_ws = 0; lead_ws <= 2; lead_ws++) {
 			for (int trail_ws = 0; trail_ws <= 2; trail_ws++) {
-				wget_buffer_printf(&buf, "%.*sx%.*sy%.*s", lead_ws, "  ", mid_ws, "  ", trail_ws, "  ");
+				wget_buffer_printf(&buf, "%*.*sx%*.*sy%*.*s",
+					lead_ws, lead_ws, "", mid_ws, mid_ws, "", trail_ws, trail_ws, "");
 				wget_buffer_trim(&buf);
 				if (!strcmp(buf.data, expected))
 					ok++;
@@ -285,6 +289,18 @@ static void test_buffer(void)
 			}
 		}
 	}
+
+	char expected[] = "";
+	for (int ws = 0; ws <= 3; ws++) {
+		wget_buffer_printf(&buf, "%*.*s", ws, ws, "");
+		wget_buffer_trim(&buf);
+		if (!strcmp(buf.data, expected))
+			ok++;
+		else {
+			failed++;
+			info_printf("test_buffer_trim: got '%s' (expected '%s') (%d)\n", buf.data, expected, ws);
+		}
+	}
 	wget_buffer_deinit(&buf);
 
 	// force reallocation
@@ -299,6 +315,12 @@ static void test_buffer(void)
 	wget_buffer_bufcpy(&buf, bufp);
 	wget_buffer_free(&bufp);
 	wget_buffer_deinit(&buf);
+
+	bufp = wget_buffer_alloc(16);
+	assert(wget_buffer_strcpy(bufp, "moin") == 4);
+	assert(wget_buffer_memset(bufp, 'A', 0) == 0);
+	assert(*bufp->data == 0);
+	wget_buffer_free(&bufp);
 }
 
 static void test_buffer_printf(void)
@@ -312,6 +334,8 @@ static void test_buffer_printf(void)
 	static const char *left_adjust[] = { "", "-" };
 	static const long long number[] = { 0, 1LL, -1LL, 10LL, -10LL, 18446744073709551615ULL };
 	static const char *modifier[] = { "", "h", "hh", "l", "ll", "z" }; // %L... won't work on OpenBSD5.0
+	enum argtype { type_int, type_long, type_long_long, type_size_t };
+	static const enum argtype modifier_type[] = { type_int, type_int, type_int, type_long, type_long_long, type_size_t };
 	static const char *conversion[] = { "d", "i", "u", "o", "x", "X" };
 	char fmt[32], result[64], string[32];
 	size_t z, a, it, n, c, m;
@@ -462,8 +486,26 @@ integer_tests:
 							#pragma GCC diagnostic push
 							#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 #endif
-							snprintf(result, sizeof(result), fmt, number[n]);
-							wget_buffer_printf(&buf, fmt, number[n]);
+							switch (modifier_type[m]) {
+							case type_int:
+								snprintf(result, sizeof(result), fmt, (int)number[n]);
+								wget_buffer_printf(&buf, fmt, (int)number[n]);
+								break;
+							case type_long:
+								snprintf(result, sizeof(result), fmt, (long)number[n]);
+								wget_buffer_printf(&buf, fmt, (long)number[n]);
+								break;
+							case type_long_long:
+								snprintf(result, sizeof(result), fmt, (long long)number[n]);
+								wget_buffer_printf(&buf, fmt, (long long)number[n]);
+								break;
+							case type_size_t:
+								snprintf(result, sizeof(result), fmt, (size_t)number[n]);
+								wget_buffer_printf(&buf, fmt, (size_t)number[n]);
+								break;
+							default:
+								abort();
+							}
 #if defined __clang__ || __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 5)
 							#pragma GCC diagnostic pop
 #endif
@@ -934,7 +976,7 @@ static void test_iri_relative_to_absolute(void)
 		const struct iri_test_data *t = &test_data[it];
 
 		base = wget_iri_parse(t->base, "utf-8");
-		wget_iri_relative_to_abs(base, t->relative, -1, &uri_buf);
+		wget_iri_relative_to_abs(base, t->relative, (size_t) -1, &uri_buf);
 
 		if (!strcmp(uri_buf.data, t->result))
 			ok++;
@@ -1030,8 +1072,8 @@ static void test_parser(void)
 		while ((dp = readdir(dirp)) != NULL) {
 			if (*dp->d_name == '.') continue;
 			if ((ext = strrchr(dp->d_name, '.'))) {
-				char fname[strlen(SRCDIR) + strlen(dp->d_name) + 8];
-				snprintf(fname, sizeof(fname), "%s/files/%s", SRCDIR, dp->d_name);
+				char fname[4096];
+				wget_snprintf(fname, sizeof(fname), "%s/files/%s", SRCDIR, dp->d_name);
 				if (!wget_strcasecmp_ascii(ext, ".xml")) {
 					info_printf("parsing %s\n", fname);
 					wget_xml_parse_file(fname, NULL, NULL, 0);
@@ -1209,7 +1251,14 @@ static void test_cookies(void)
 	int result, result_psl;
 
 	cookies = wget_cookie_db_init(NULL);
-	wget_cookie_db_load_psl(cookies, SRCDIR "/files/public_suffix_list.dat");
+
+	if (wget_cookie_db_load_psl(cookies, SRCDIR "/files/public_suffix_list.dat") == -1) {
+#ifdef WITH_LIBPSL
+			failed++;
+			info_printf("Failed to load %s (errno=%d)\n", SRCDIR "/files/public_suffix_list.dat", errno);
+#endif
+			goto out;
+	}
 
 	for (it = 0; it < countof(test_data); it++) {
 		char *header, *set_cookie;
@@ -1272,6 +1321,7 @@ next:
 		wget_iri_free(&iri);
 	}
 
+out:
 	wget_cookie_db_free(&cookies);
 }
 
@@ -1307,8 +1357,8 @@ static void test_hsts(void)
 		{ "www.example.com", 8080, 0 }, // wrong port
 	};
 	wget_hsts_db *hsts_db = wget_hsts_db_init(NULL, NULL);
-	time_t maxage;
-	char include_subdomains;
+	int64_t maxage;
+	bool include_subdomains;
 	int n;
 
 	// fill HSTS database with values
@@ -1382,7 +1432,7 @@ static void test_hpkp(void)
 		{ "www.example3.com", 443, "max-age=14400" }, // this removes the previous entry, due to no PINs
 	};
 	struct hpkp_db_params {
-		time_t
+		int64_t
 			maxage;
 		bool
 			include_subdomains;
@@ -1607,7 +1657,7 @@ static void test_utils(void)
 		for (it = 0; it <= 255; it++) {
 			src[0] = (unsigned char) it;
 			wget_memtohex(src, 1, dst1, ndst);
-			snprintf(dst2, ndst, "%02x", src[0]);
+			wget_snprintf(dst2, ndst, "%02x", src[0]);
 			if (strcmp(dst1, dst2)) {
 				info_printf("buffer_to_hex failed: '%s' instead of '%s' (ndst=%d)\n", dst1, dst2, ndst);
 				failed++;
@@ -1878,7 +1928,7 @@ static void test_stringmap(void)
 		} else ok++;
 
 		iter = wget_stringmap_iterator_alloc(m);
-		for (it = 0; (skey = wget_stringmap_iterator_next(iter, &val)); it++) {
+		for (it = 0; (skey = wget_stringmap_iterator_next(iter, (void **) &val)); it++) {
 			int x = atoi(skey + 30), y = atoi(val);
 
 			if (!(c_isdigit(*val) && x >= 0 && x == y)) {
@@ -1929,7 +1979,7 @@ static void test_stringmap(void)
 
 		// now, remove every single entry
 		for (it = 0; it < 26; it++) {
-			snprintf(keybuf, sizeof(keybuf), "http://www.example.com/subdir/%d.html", it);
+			wget_snprintf(keybuf, sizeof(keybuf), "http://www.example.com/subdir/%d.html", it);
 			wget_stringmap_remove(m, keybuf);
 		}
 
@@ -2139,9 +2189,22 @@ static void test_netrc(void)
 			3,
 			{
 				{ "localhost", "theuser", "thepw" },
-				{ "localhost2", "theuser2", "thepw2" }
+				{ "localhost2", "theuser2", "thepw2" },
+				{ "abc", "111", "222" }
 			}
 		},
+		{ "machine m\nlogin u\npassword a\\b", 1,       {{ "m", "u", "ab" }} },
+		{ "machine m\nlogin u\npassword a\\\\b", 1,     {{ "m", "u", "a\\b" }} },
+		{ "machine m\nlogin u\npassword \"a\\\\b\"", 1, {{ "m", "u", "a\\b" }} },
+		{ "machine m\nlogin u\npassword \"a\\\"b\"", 1, {{ "m", "u", "a\"b" }} },
+		{ "machine m\nlogin u\npassword a\"b", 1,       {{ "m", "u", "a\"b" }} },
+		{ "machine m\nlogin u\npassword a\\\\\\\\b", 1, {{ "m", "u", "a\\\\b" }} },
+		{ "machine m\nlogin u\npassword a\\\\", 1,      {{ "m", "u", "a\\" }} },
+		{ "machine m\nlogin u\npassword \"a\\\\\"", 1,  {{ "m", "u", "a\\" }} },
+		{ "machine m\nlogin u\npassword a\\", 1,        {{ "m", "u", "a" }} },
+		{ "machine m\nlogin u\npassword \"a b\"", 1,    {{ "m", "u", "a b" }} },
+		{ "machine m\nlogin u\npassword a b", 1,        {{ "m", "u", "a" }} },
+		{ "machine m\nlogin u\npassword a\\ b", 1,        {{ "m", "u", "a b" }} },
 	};
 	FILE *fp;
 	wget_netrc_db *netrc_db;
@@ -2177,11 +2240,11 @@ static void test_netrc(void)
 					failed++;
 				}
 				else if (strcmp(netrc->login, e->login)) {
-					info_printf("[%u] Login mismatch '%s' / '%s' in netrc_db\n", it, netrc->login, e->login);
+					info_printf("[%u] Login mismatch in netrc_db: expected '%s', got '%s'\n", it, e->login, netrc->login);
 					failed++;
 				}
 				else if (strcmp(netrc->password, e->password)) {
-					info_printf("[%u] Password mismatch '%s' / '%s' in netrc_db\n", it, netrc->login, e->login);
+					info_printf("[%u] Password mismatch in netrc_db: expected '%s', got '%s'\n", it, e->password, netrc->password);
 					failed++;
 				} else
 					ok++;
@@ -2200,42 +2263,52 @@ static void test_robots(void)
 {
 	static const struct test_data {
 		const char *
-			data;
+			name;
+		const char *
+			input;
 		const char *
 			path[3];
 		const char *
 			sitemap[3];
 	} test_data[] = {
 		{
-			// Deny all robots from part of the server
+			"Allow all + sitemap",
 			"User-agent: *\n"
-			"Disallow: /cgi-bin/\n",
-			{ "/cgi-bin/", NULL },
-			{ "", NULL }
+			"Disallow: # allow all\n"
+			"Sitemap: http://www.example.com/sitemap.xml",
+			{ NULL },
+			{ "http://www.example.com/sitemap.xml", NULL }
 		},
 		{
-			// Deny all robots from the entire server
+			"Deny all /cgi-bin",
+			"User-agent: *\n"
+			"Disallow: /cgi-bin/ # comment\n",
+			{ "/cgi-bin/", NULL },
+			{ NULL }
+		},
+		{
+			"Deny all /",
 			"User-agent: *\n"
 			"Disallow: /\n",
 			{ "/", NULL },
 			{ NULL }
 		},
 		{
-			// allow a single robot for part of the server
+			"Deny wget2 /cgi/bin",
 			"User-agent: wget2\n"
 			"Disallow: /cgi-bin/\n",
 			{ "/cgi-bin/", NULL },
 			{ NULL }
 		},
 		{
-			// deny a single robot
+			"Deny wget2 /",
 			"User-agent: wget2\n"
 			"Disallow: /\n",
 			{ "/", NULL },
 			{ NULL }
 		},
 		{
-			// allow a single robot
+			"Allow all but deny wget2 /",
 			"User-agent: *\n"
 			"Disallow: /\n"
 			"User-agent: wget2\n"
@@ -2244,14 +2317,14 @@ static void test_robots(void)
 			{ NULL }
 		},
 		{
-			// Allow all robots complete access
-			// or simply don't use a robots.txt at all
+			"Allow all",
 			"User-agent: *\n"
 			"Disallow: \n",
 			{ NULL },
 			{ NULL }
 		},
 		{
+			"Deny all /cgi-bin + sitemap",
 			// with 1 sitemap
 			"User-agent: *\n"
 			"Disallow: /cgi-bin/\n"
@@ -2260,7 +2333,7 @@ static void test_robots(void)
 			{ "http://www.example.com/sitemap.xml", NULL }
 		},
 		{
-			// with 2 sitemap
+			"Deny all /cgi-bin + 2 sitemaps",
 			"User-agent: *\n"
 			"Disallow: /cgi-bin/\n"
 			"Sitemap: http://www.example1.com/sitemap.xml\n"
@@ -2269,7 +2342,7 @@ static void test_robots(void)
 			{ "http://www.example1.com/sitemap.xml", "http://www.example2.com/sitemap.xml", NULL }
 		},
 		{
-			// with 2 "Disallow" entries
+			"Deny all /cgi-bin, /tmp + 2 sitemaps",
 			"User-agent: *\n"
 			"Disallow: /cgi-bin/\n"
 			"Disallow: /tmp/\n"
@@ -2279,7 +2352,7 @@ static void test_robots(void)
 			{ "http://www.example1.com/sitemap.xml", "http://www.example2.com/sitemap.xml", NULL }
 		},
 		{
-			// with equal number entries
+			"Deny all /cgi-bin, /tmp, /jumk + 3 sitemaps",
 			"User-agent: *\n"
 			"Disallow: /cgi-bin/\n"
 			"Disallow: /tmp/\n"
@@ -2291,27 +2364,41 @@ static void test_robots(void)
 			{ "http://www.example1.com/sitemap.xml", "http://www.example2.com/sitemap.xml", "http://www.example3.com/sitemap.xml" }
 		},
 		{
-			// null termination test
+			"Missing EOL",
 			"User-agent: *\n"
 			"Disallow: /cgi-bin/",
 			{ "/cgi-bin/", NULL },
-			{ "", NULL }
+			{ NULL }
 		},
-		{
-			// null termination test #2
-			"User-agent: *"
-			"Disallow: /cgi-bin/",
-			{ "/cgi-bin/", NULL },
-			{ "", NULL }
-		}
 	};
 
 	for (unsigned it = 0; it < countof(test_data); it++) {
 		const struct test_data *t = &test_data[it];
 		wget_robots *robots;
+		int count;
 
-		if (wget_robots_parse(&robots, t->data, PACKAGE_NAME) != WGET_E_SUCCESS) {
-			info_printf("Failed to parse: \"%s\" on robots\n", t->path[it]);
+		if (wget_robots_parse(&robots, t->input, PACKAGE_NAME) != WGET_E_SUCCESS) {
+			info_printf("'%s': Failed to parse input\n", t->name);
+			failed++;
+			continue;
+		}
+
+		count = 0;
+		for (unsigned it2 = 0; it2 < countof(test_data[it].path) && t->path[it2]; it2++, count++)
+			;
+		if (count != wget_robots_get_path_count(robots)) {
+			info_printf("'%s': paths mismatch: expected %d, got %d\n",
+				t->name, count, wget_robots_get_path_count(robots));
+			failed++;
+			continue;
+		}
+
+		count = 0;
+		for (unsigned it2 = 0; it2 < countof(test_data[it].sitemap) && t->sitemap[it2]; it2++, count++)
+			;
+		if (count != wget_robots_get_sitemap_count(robots)) {
+			info_printf("'%s': sitemap # mismatch: expected %d, got %d\n",
+				t->name, count, wget_robots_get_sitemap_count(robots));
 			failed++;
 			continue;
 		}
@@ -2325,7 +2412,7 @@ static void test_robots(void)
 					it3 = n;
 					ok++;
 				} else if ((strcmp(paths->p, t->path[it2]) && it3 == n - 1)) {
-					info_printf("Cannot find path: \"%s\" on robots\n", t->path[it2]);
+					info_printf("'%s': Cannot find path: \"%s\" on robots\n", t->name, t->path[it2]);
 					failed++;
 				}
 			}
@@ -2340,7 +2427,7 @@ static void test_robots(void)
 					it3 = n;
 					ok++;
 				} else if ((strcmp(sitemaps, t->sitemap[it2]) && it3 == n - 1)) {
-					info_printf("Cannot find sitemap: \"%s\" on robots\n", t->sitemap[it2]);
+					info_printf("'%s': Cannot find sitemap: \"%s\" on robots\n", t->name, t->sitemap[it2]);
 					failed++;
 				}
 			}
@@ -2398,6 +2485,70 @@ static void test_set_proxy(void)
 	}
 }
 
+
+static void test_match_no_proxy(void)
+{
+	static const struct test_data {
+		const char *
+			no_proxy;
+		const char *
+			hostip;
+		const char *
+			encoding;
+		int
+			result;
+	} test_data[] = {
+		{ "10.250.192.78/12", "142.251.33.101", NULL, 0},
+		{ "142.250.192.78/12", "142.251.33.101", NULL, 1},
+		{ "142.250.192.78/50", "142.251.33.101", NULL, 0},
+		{ "10.250.192.78.123/12", "142.251.33.101", NULL, 0},
+		{ "142.250.192.78/32", "142.250.180.101", NULL, 0},
+		{ "142.250.192.78/0", "142.250.180.101", NULL, 1},
+		{ "142.250.192.78/-1", "142.250.180.101", NULL, 0},
+		{ "", "142.250.180.101", NULL, 0},
+		{ "142.251.33.101,10.250.192.78/12", "142.251.33.101", NULL, 1},
+		{ "10.250.192.78/12, 142.251.33.101", "142.251.33.101", NULL, 1},
+		{ "10.250.192.78/12, 142.251.33.101", "142.251.33.101", NULL, 1},
+		{ "2402:9400:1234:5670::", "2402:9400:1234:5670::", NULL, 1},
+		{ "2402:9400:1234:5670::", "2402:9400:1234:5671::", NULL, 0},
+		{ "2402:9400:1234:5670::/60", "2402:9400:1234:5678::", NULL, 1},
+		{ "2402:9400:1234:5670::/60", "2402:9400:1234:5680::", NULL, 0}
+	};
+
+	for (unsigned it = 0; it < countof(test_data); it++) {
+		const struct test_data *t = &test_data[it];
+		wget_http_set_no_proxy(t->no_proxy, t->encoding);
+		const wget_vector *no_proxies = wget_http_get_no_proxy();
+		int n = wget_http_match_no_proxy(no_proxies, t->hostip);
+
+		if (n == t->result) {
+			ok++;
+		} else {
+			failed++;
+			info_printf("Failed [%u]: wget_http_match_no_proxy(\"%s\",\"%s\") -> %d (expected %d)\n",
+				it, t->no_proxy, t->hostip, n, t->result);
+		}
+	}
+}
+
+// Add some corner cases here.
+static void test_parse_header_line(void)
+{
+	const char *filename;
+
+	// from https://github.com/rockdaboot/wget2/issues/235
+	wget_http_parse_content_disposition("attachment; filename=\"with space\"", &filename);
+
+	if (strcmp(filename, "with space") == 0) {
+		ok++;
+	} else {
+		failed++;
+		info_printf("HTTP keep-alive Connection header could not be set.\n");
+	}
+
+	xfree(filename);
+}
+
 static void test_parse_response_header(void)
 {
 	char *response_text = wget_strdup(
@@ -2480,14 +2631,14 @@ int main(int argc, const char **argv)
 		// fallthrough
 	}
 	else if (!strcmp(valgrind, "1")) {
-		char cmd[strlen(argv[0]) + 256];
+		char cmd[4096];
 
-		snprintf(cmd, sizeof(cmd), "VALGRIND_TESTS=\"\" valgrind --error-exitcode=301 --leak-check=yes --show-reachable=yes --track-origins=yes %s", argv[0]);
+		wget_snprintf(cmd, sizeof(cmd), "VALGRIND_TESTS=\"\" valgrind --error-exitcode=301 --leak-check=yes --show-reachable=yes --track-origins=yes %s", argv[0]);
 		return system(cmd) != 0;
 	} else {
-		char cmd[strlen(valgrind) + strlen(argv[0]) + 32];
+		char cmd[4096];
 
-		snprintf(cmd, sizeof(cmd), "VALGRIND_TESTS="" %s %s", valgrind, argv[0]);
+		wget_snprintf(cmd, sizeof(cmd), "VALGRIND_TESTS="" %s %s", valgrind, argv[0]);
 		return system(cmd) != 0;
 	}
 
@@ -2525,7 +2676,7 @@ int main(int argc, const char **argv)
 	test_iri_relative_to_absolute();
 	test_iri_compare();
 	test_parser();
-
+	test_match_no_proxy();
 	test_cookies();
 	test_hsts();
 	test_hpkp();
@@ -2535,6 +2686,7 @@ int main(int argc, const char **argv)
 	test_robots();
 	test_set_proxy();
 	test_parse_response_header();
+	test_parse_header_line();
 
 	selftest_options() ? failed++ : ok++;
 
